﻿using System;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 폰트 다이얼로그
    /// </summary>
    public partial class FontDialog : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 업데이트 보류 여부
        /// </summary>
        private bool isUpdateSuppressed = true;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 타입 페이스 - Typeface

        /// <summary>
        /// 타입 페이스
        /// </summary>
        public Typeface Typeface
        {
            get
            {
                ListItem fontFamilyListItem = this.fontFamilySingleListBox.GetItem();

                if(fontFamilyListItem == null)
                {
                    return null;
                }

                ListItem fontStyleListItem = this.fontStyleSingleListBox.GetItem();

                if(fontStyleListItem == null)
                {
                    return null;
                }

                ListItem fontWeightListItem = this.fontWeightSingleListBox.GetItem();

                if(fontWeightListItem == null)
                {
                    return null;
                }

                ListItem fontStretchListItem = this.fontStretchSingleListBox.GetItem();

                if(fontStretchListItem == null)
                {
                    return null;
                }

                Typeface typeface = new Typeface
                (
                    fontFamilyListItem.Tag as FontFamily,
                    (FontStyle)fontStyleListItem.Tag,
                    (FontWeight)fontWeightListItem.Tag,
                    (FontStretch)fontStretchListItem.Tag
                );

                return typeface;
            }
            set
            {
                SetSingleListBoxValue(this.fontFamilySingleListBox, UIHelper.GetFontName(value.FontFamily));

                this.isUpdateSuppressed = true;

                SetSingleListBoxValue(this.fontStyleSingleListBox  , value.Style.ToString()  );
                SetSingleListBoxValue(this.fontWeightSingleListBox , value.Weight.ToString() );
                SetSingleListBoxValue(this.fontStretchSingleListBox, value.Stretch.ToString());
                
                this.isUpdateSuppressed = false;

                UpdateSample();
            }
        }

        #endregion
        #region 폰트 크기 - FaceSize

        /// <summary>
        /// 폰트 크기
        /// </summary>
        /// <remarks>상속받은 Window 클래스에 FontSize 속성이 있기 때문이다.</remarks>
        public double FaceSize
        {
            get
            {
                ListItem item = this.fontSizeSingleListBox.GetItem();

                double size;

                if(item == null)
                {
                    size = 8.25d;
                }
                else
                {
                    size = (double)item.Tag;
                }

                return size / 0.75d;
            }
            set
            {
                double size = 0.75d * value;

                string sizeString = size.ToString();

                if(!this.fontSizeSingleListBox.Contains(sizeString))
                {
                    ObservableCollection<ListItem> collection = this.fontSizeSingleListBox.ItemsSource as ObservableCollection<ListItem>;

                    if(collection == null)
                    {
                        collection = new ObservableCollection<ListItem>();

                        this.fontSizeSingleListBox.ItemsSource = collection;
                    }

                    ListItem item = new ListItem(sizeString);

                    item.Tag = size;

                    collection.Insert(0, item);
                }

                this.fontSizeSingleListBox.Value = sizeString;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - FontDialog()

        /// <summary>
        /// 생성자
        /// </summary>
        public FontDialog()
        {
            InitializeComponent();

            this.fontFamilySingleListBox.ValueChanged  += fontFamilySingleListBox_ValueChanged;
            this.fontStyleSingleListBox.ValueChanged   += fontStyleSingleListBox_ValueChanged;
            this.fontWeightSingleListBox.ValueChanged  += fontWeightSingleListBox_ValueChanged;
            this.fontStretchSingleListBox.ValueChanged += fontStretchSingleListBox_ValueChanged;
            this.fontSizeSingleListBox.ValueChanged    += fontSizeSingleListBox_ValueChanged;

            this.okButton.Click     += okButton_Click;
            this.cancelButton.Click += cancelButton_Click;

            SetFontFamilySingleListBoxData(string.Empty);

            SetFontSizeSingleListBoxData(string.Empty);

            this.fontFamilySingleListBox.Focus();

            this.isUpdateSuppressed = false;

            UpdateSample();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 폰트 패밀리 싱글 리스트 박스 값 변경시 처리하기 - fontFamilySingleListBox_ValueChanged(sender, e)

        /// <summary>
        /// 폰트 패밀리 싱글 리스트 박스 값 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void fontFamilySingleListBox_ValueChanged(object sender, EventArgs e)
        {
            string fontStyleValue   = this.fontStyleSingleListBox.Value;
            string fontWeightValue  = this.fontWeightSingleListBox.Value;
            string fontStretchValue = this.fontStretchSingleListBox.Value;

            this.isUpdateSuppressed = true;

            ListItem fontFamilyListItem = this.fontFamilySingleListBox.GetItem();

            SetFontStyleSingleListBoxData  (fontFamilyListItem == null ? null : fontFamilyListItem.Tag as FontFamily, fontStyleValue  );
            SetFontWeightSingleListBoxData (fontFamilyListItem == null ? null : fontFamilyListItem.Tag as FontFamily, fontWeightValue );
            SetFontStretchSingleListBoxData(fontFamilyListItem == null ? null : fontFamilyListItem.Tag as FontFamily, fontStretchValue);

            this.isUpdateSuppressed = false;

            UpdateSample();
        }

        #endregion
        #region 폰트 스타일 싱글 리스트 박스 값 변경시 처리하기 - fontStyleSingleListBox_ValueChanged(sender, e)

        /// <summary>
        /// 폰트 스타일 싱글 리스트 박스 값 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void fontStyleSingleListBox_ValueChanged(object sender, EventArgs e)
        {
            UpdateSample();
        }

        #endregion
        #region 폰트 웨이트 싱글 리스트 박스 값 변경시 처리하기 - fontWeightSingleListBox_ValueChanged(sender, e)

        /// <summary>
        /// 폰트 웨이트 싱글 리스트 박스 값 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void fontWeightSingleListBox_ValueChanged(object sender, EventArgs e)
        {
            UpdateSample();
        }

        #endregion
        #region 폰트 스트레치 싱글 리스트 박스 값 변경시 처리하기 - fontStretchSingleListBox_ValueChanged(sender, e)

        /// <summary>
        /// 폰트 스트레치 싱글 리스트 박스 값 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void fontStretchSingleListBox_ValueChanged(object sender, EventArgs e)
        {
            UpdateSample();
        }

        #endregion
        #region 폰트 크기 싱글 리스트 박스 값 변경시 처리하기 - fontSizeSingleListBox_ValueChanged(sender, e)

        /// <summary>
        /// 폰트 크기 싱글 리스트 박스 값 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void fontSizeSingleListBox_ValueChanged(object sender, EventArgs e)
        {
            UpdateSample();
        }

        #endregion
        #region OK 버튼 클릭시 처리하기 - okButton_Click(sender, e)

        /// <summary>
        /// OK 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void okButton_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = true;
        }

        #endregion
        #region Cancel 버튼 클릭시 처리하기 - cancelButton_Click(sender, e)

        /// <summary>
        /// Cancel 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void cancelButton_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = false;
        }

        #endregion

        #region 싱글 리스트 박스 값 설정하기 - SetSingleListBoxValue(singleListBox, value)

        /// <summary>
        /// 싱글 리스트 박스 값 설정하기
        /// </summary>
        /// <param name="singleListBox">싱글 리스트 박스</param>
        /// <param name="value">값</param>
        private void SetSingleListBoxValue(SingleListBox singleListBox, string value)
        {
            ObservableCollection<ListItem> collection = singleListBox.ItemsSource as ObservableCollection<ListItem>;

            if(collection == null || collection.Count == 0)
            {
                return;
            }

            if(singleListBox.Contains(value))
            {
                singleListBox.Value = value;
            }
            else
            {
                singleListBox.Value = singleListBox.GetItem(0).Value;
            }
        }

        #endregion

        #region 폰트 패밀리 싱글 리스트 박스 데이터 설정하기 - SetFontFamilySingleListBoxData(value)

        /// <summary>
        /// 폰트 패밀리 싱글 리스트 박스 데이터 설정하기
        /// </summary>
        /// <param name="value">값</param>
        private void SetFontFamilySingleListBoxData(string value)
        {
            this.fontFamilySingleListBox.ItemsSource = UIHelper.GetFontFamilyData();

            SetSingleListBoxValue(this.fontFamilySingleListBox, value);
        }

        #endregion
        #region 폰트 스타일 싱글 리스트 박스 데이터 설정하기 - SetFontStyleSingleListBoxData(fontFamily, value)

        /// <summary>
        /// 폰트 스타일 싱글 리스트 박스 데이터 설정하기
        /// </summary>
        /// <param name="fontFamily">폰트 패밀리</param>
        /// <param name="value">값</param>
        private void SetFontStyleSingleListBoxData(FontFamily fontFamily, string value)
        {
            if(fontFamily == null)
            {
                this.fontStyleSingleListBox.ItemsSource = null;
            }
            else
            {
                this.fontStyleSingleListBox.ItemsSource = UIHelper.GetFontStyleData(fontFamily);

                SetSingleListBoxValue(this.fontStyleSingleListBox, value);
            }
        }

        #endregion
        #region 폰트 웨이트 싱글 리스트 박스 데이터 설정하기 - SetFontWeightSingleListBoxData(fontFamily, value)

        /// <summary>
        /// 폰트 웨이트 싱글 리스트 박스 데이터 설정하기
        /// </summary>
        /// <param name="fontFamily">폰트 패밀리</param>
        /// <param name="value">값</param>
        private void SetFontWeightSingleListBoxData(FontFamily fontFamily, string value)
        {
            if(fontFamily == null)
            {
                this.fontWeightSingleListBox.ItemsSource = null;
            }
            else
            {
                this.fontWeightSingleListBox.ItemsSource = UIHelper.GetFontWeightData(fontFamily);

                SetSingleListBoxValue(this.fontWeightSingleListBox, value);
            }
        }

        #endregion
        #region 폰트 스트레치 싱글 리스트 박스 데이터 설정하기 - SetFontStretchSingleListBoxData(fontFamily, value)

        /// <summary>
        /// 폰트 스트레치 싱글 리스트 박스 데이터 설정하기
        /// </summary>
        /// <param name="fontFamily">폰트 패밀리</param>
        /// <param name="value">값</param>
        private void SetFontStretchSingleListBoxData(FontFamily fontFamily, string value)
        {
            if(fontFamily == null)
            {
                this.fontStretchSingleListBox.ItemsSource = null;
            }
            else
            {
                this.fontStretchSingleListBox.ItemsSource = UIHelper.GetFontStretchData(fontFamily);

                SetSingleListBoxValue(this.fontStretchSingleListBox, value);
            }
        }

        #endregion
        #region 폰트 크기 싱글 리스트 박스 데이터 설정하기 - SetFontSizeSingleListBoxData(value)

        /// <summary>
        /// 폰트 크기 싱글 리스트 박스 데이터 설정하기
        /// </summary>
        /// <param name="value">값</param>
        private void SetFontSizeSingleListBoxData(string value)
        {
            this.fontSizeSingleListBox.ItemsSource = UIHelper.GetFontSizeData();

            SetSingleListBoxValue(this.fontSizeSingleListBox, value);
        }

        #endregion

        #region 샘플 갱신하기 - UpdateSample()

        /// <summary>
        /// 샘플 갱신하기
        /// </summary>
        private void UpdateSample()
        {
            if(this.isUpdateSuppressed)
            {
                return;
            }

            ListItem fontFamilyListItem = this.fontFamilySingleListBox.GetItem();

            if(fontFamilyListItem == null)
            {
                this.displayLabel.Content = null;

                return;
            }

            ListItem fontStyleListItem = this.fontStyleSingleListBox.GetItem();

            if(fontStyleListItem == null)
            {
                this.displayLabel.Content = null;

                return;
            }

            ListItem fontWeightListItem = this.fontWeightSingleListBox.GetItem();

            if(fontWeightListItem == null)
            {
                this.displayLabel.Content = null;

                return;
            }

            ListItem fontStretchListItem = this.fontStretchSingleListBox.GetItem();

            if(fontStretchListItem == null)
            {
                this.displayLabel.Content = null;

                return;
            }

            ListItem fontSizeListItem = this.fontSizeSingleListBox.GetItem();

            if(fontSizeListItem == null)
            {
                this.displayLabel.Content = null;

                return;
            }

            this.displayLabel.Content = "가나다라마 ABC abc 12345";

            this.displayLabel.FontFamily  = fontFamilyListItem.Tag as FontFamily;
            this.displayLabel.FontStyle   = (FontStyle)fontStyleListItem.Tag;
            this.displayLabel.FontWeight  = (FontWeight)fontWeightListItem.Tag;
            this.displayLabel.FontStretch = (FontStretch)fontStretchListItem.Tag;
            this.displayLabel.FontSize    = (double)fontSizeListItem.Tag / 0.75d;
        }

        #endregion
    }
}