﻿using System.Windows;
using System.Windows.Controls.Primitives;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 아이콘 버튼
    /// </summary>
    public partial class IconButton : ToggleButton
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 아이콘 의존 속성
        /// </summary>
        public static readonly DependencyProperty IconProperty;

        /// <summary>
        /// 화살표 표시 여부 의존 속성
        /// </summary>
        public static readonly DependencyProperty ArrowVisibilityProperty;

        /// <summary>
        /// 그룹명 의존 속성
        /// </summary>
        public static readonly DependencyProperty GroupNameProperty;

        /// <summary>
        /// 값 의존 속성
        /// </summary>
        public static readonly DependencyProperty ValueProperty;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 아이콘 - Icon

        /// <summary>
        /// 아이콘
        /// </summary>
        public ImageSource Icon
        {
            get
            {
                return (ImageSource)GetValue(IconProperty);
            }
            set
            {
                SetValue(IconProperty, value);
            }
        }

        #endregion
        #region 화살표 표시 여부 - ArrowVisibility

        /// <summary>
        /// 화살표 표시 여부
        /// </summary>
        public Visibility ArrowVisibility
        {
            get
            {
                return (Visibility)GetValue(ArrowVisibilityProperty);
            }
            set
            {
                SetValue(ArrowVisibilityProperty, value);
            }
        }

        #endregion
        #region 그룹명 - GroupName

        /// <summary>
        /// 그룹명
        /// </summary>
        public string GroupName
        {
            get
            {
                return (string)GetValue(GroupNameProperty);
            }
            set
            {
                SetValue(GroupNameProperty, value);
            }
        }

        #endregion
        #region 값 - Value

        /// <summary>
        /// 값
        /// </summary>
        public string Value
        {
            get
            {
                return (string)GetValue(ValueProperty);
            }
            set
            {
                SetValue(ValueProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - IconButton()

        /// <summary>
        /// 생성자
        /// </summary>
        static IconButton()
        {
            IconProperty            = UIHelper.RegisterDependencyProperty(null              , false, false, false, true, null, "Icon"           , typeof(ImageSource), typeof(IconButton), null);
            ArrowVisibilityProperty = UIHelper.RegisterDependencyProperty(Visibility.Visible, false, false, false, true, null, "ArrowVisibility", typeof(Visibility) , typeof(IconButton), null);
            GroupNameProperty       = UIHelper.RegisterDependencyProperty(string.Empty      , false, false, false, true, null, "GroupName"      , typeof(string)     , typeof(IconButton), null);
            ValueProperty           = UIHelper.RegisterDependencyProperty(string.Empty      , false, false, false, true, null, "Value"          , typeof(string)     , typeof(IconButton), null);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - IconButton()

        /// <summary>
        /// 생성자
        /// </summary>
        public IconButton()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 클릭 실행하기 - PerformClick()

        /// <summary>
        /// 클릭 실행하기
        /// </summary>
        public void PerformClick()
        {
            RaiseEvent(new RoutedEventArgs(ButtonBase.ClickEvent));
        }

        #endregion
    }
}