﻿using System;
using System.Collections;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 아이콘 팝업 버튼
    /// </summary>
    public partial class IconPopupButton : UserControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값 변경시 - ValueChanged

        /// <summary>
        /// 값 변경시
        /// </summary>
        public event EventHandler ValueChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 항목 소스 의존 속성
        /// </summary>
        public static readonly DependencyProperty ItemsSourceProperty;

        /// <summary>
        /// 항목 패널 의존 속성
        /// </summary>
        public static readonly DependencyProperty ItemsPanelProperty;

        /// <summary>
        /// 아이콘 의존 속성
        /// </summary>
        public static readonly DependencyProperty IconProperty;

        /// <summary>
        /// 화살표 표시 여부 의존 속성
        /// </summary>
        public static readonly DependencyProperty ArrowVisibilityProperty;

        /// <summary>
        /// 그룹명 의존 속성
        /// </summary>
        public static readonly DependencyProperty GroupNameProperty;

        /// <summary>
        /// 값 의존 속성
        /// </summary>
        public static readonly DependencyProperty ValueProperty;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 첫번째 로드 여부
        /// </summary>
        private bool firstLoaded = true;

        /// <summary>
        /// 마우스 버튼 상위 여부
        /// </summary>
        private bool isMouseOverButton = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 소스 - ItemsSource

        /// <summary>
        /// 항목 소스
        /// </summary>
        public IEnumerable ItemsSource
        {
            get
            {
                return (IEnumerable)GetValue(ItemsSourceProperty);
            }
            set
            {
                SetValue(ItemsSourceProperty, value);
            }
        }

        #endregion
        #region 항목 패널 - ItemsPanel

        /// <summary>
        /// 항목 패널
        /// </summary>
        public ItemsPanelTemplate ItemsPanel
        {
            get
            {
                return (ItemsPanelTemplate)GetValue(ItemsPanelProperty);
            }
            set
            {
                SetValue(ItemsPanelProperty, value);
            }
        }

        #endregion
        #region 아이콘 - Icon

        /// <summary>
        /// 아이콘
        /// </summary>
        public ImageSource Icon
        {
            get
            {
                return (ImageSource)GetValue(IconProperty);
            }
            set
            {
                SetValue(IconProperty, value);
            }
        }

        #endregion
        #region 화살표 표시 여부 - ArrowVisibility

        /// <summary>
        /// 화살표 표시 여부
        /// </summary>
        public Visibility ArrowVisibility
        {
            get
            {
                return (Visibility)GetValue(ArrowVisibilityProperty);
            }
            set
            {
                SetValue(ArrowVisibilityProperty, value);
            }
        }

        #endregion
        #region 그룹명 - GroupName

        /// <summary>
        /// 그룹명
        /// </summary>
        public string GroupName
        {
            get
            {
                return (string)GetValue(GroupNameProperty);
            }
            set
            {
                SetValue(GroupNameProperty, value);
            }
        }

        #endregion
        #region 값 - Value

        /// <summary>
        /// 값
        /// </summary>
        public string Value
        {
            get
            {
                return (string)GetValue(ValueProperty);
            }
            set
            {
                SetValue(ValueProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - IconPopupButton()

        /// <summary>
        /// 생성자
        /// </summary>
        static IconPopupButton()
        {
            ItemsSourceProperty     = UIHelper.RegisterDependencyProperty(null              , false, false, false, true, null                        , "ItemsSource"    , typeof(IEnumerable       ), typeof(IconPopupButton), null);
            ItemsPanelProperty      = UIHelper.RegisterDependencyProperty(null              , false, false, false, true, null                        , "ItemsPanel"     , typeof(ItemsPanelTemplate), typeof(IconPopupButton), null);
            IconProperty            = UIHelper.RegisterDependencyProperty(null              , false, false, false, true, null                        , "Icon"           , typeof(ImageSource       ), typeof(IconPopupButton), null);
            ArrowVisibilityProperty = UIHelper.RegisterDependencyProperty(Visibility.Visible, false, false, false, true, null                        , "ArrowVisibility", typeof(Visibility        ), typeof(IconPopupButton), null);
            GroupNameProperty       = UIHelper.RegisterDependencyProperty(string.Empty      , false, false, false, true, null                        , "GroupName"      , typeof(string            ), typeof(IconPopupButton), null);
            ValueProperty           = UIHelper.RegisterDependencyProperty(string.Empty      , false, false, false, true, valuePropertyChangedCallback, "Value"          , typeof(string            ), typeof(IconPopupButton), null);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - IconPopupButton()

        /// <summary>
        /// 생성자
        /// </summary>
        public IconPopupButton()
        {
            InitializeComponent();

            UIHelper.SetBinding(this, ItemsSourceProperty, this.itemsControl, ItemsControl.ItemsSourceProperty, BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);
            UIHelper.SetBinding(this, ItemsPanelProperty , this.itemsControl, ItemsControl.ItemsPanelProperty , BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);

            this.popup.IsOpen = true;
            this.popup.IsOpen = false;

            this.Loaded           += iconPopupButton_Loaded;
            this.iconButton.Click += iconButton_Click;
            this.popup.Closed     += popup_Closed;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 값 속성 변경시 콜백 처리하기 - valuePropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 값 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void valuePropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            IconPopupButton iconPopupButton = dependencyObject as IconPopupButton;


            string newValue = e.NewValue == null ? string.Empty : e.NewValue as string;

            IEnumerable enumerable = iconPopupButton.ItemsSource as IEnumerable;

            if(enumerable != null)
            {
                foreach(object child in enumerable)
                {
                    IconButton childIconButton = child as IconButton;

                    if(childIconButton != null)
                    {
                        try
                        {
                            childIconButton.Click -= iconPopupButton.childIconButton_Click;

                            if(childIconButton.Value.Equals(newValue))
                            {
                                childIconButton.IsChecked = true;

                                iconPopupButton.Icon    = childIconButton.Icon;
                                iconPopupButton.ToolTip = childIconButton.ToolTip;
                            }
                            else
                            {
                                childIconButton.IsChecked = false;
                            }
                        }
                        finally
                        {
                            childIconButton.Click += iconPopupButton.childIconButton_Click;
                        }
                    }
                }
            }


            iconPopupButton.FireValueChanged();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 값 변경시 이벤트 발생시키기 - FireValueChanged()

        /// <summary>
        /// 값 변경시 이벤트 발생시키기
        /// </summary>
        protected void FireValueChanged()
        {
            if(ValueChanged != null)
            {
                ValueChanged(this, EventArgs.Empty);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 아이콘 팝업 버튼 로드시 처리하기 - iconPopupButton_Loaded(object sender, RoutedEventArgs e)

        /// <summary>
        /// 아이콘 팝업 버튼 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void iconPopupButton_Loaded(object sender, RoutedEventArgs e)
        {
            if(this.firstLoaded)
            {
                this.firstLoaded = false;
            }
            else
            {
                return;
            }

            if(ItemsSource != null)
            {
                IEnumerable enumerable = ItemsSource as IEnumerable;

                foreach(object child in enumerable)
                {
                    IconButton childIconButton = child as IconButton;

                    if(childIconButton != null)
                    {
                        childIconButton.Click += childIconButton_Click;
                    }
                }
            }
        }

        #endregion
        #region 아이콘 버튼 클릭시 처리하기 - iconButton_Click(sender, e)

        /// <summary>
        /// 아이콘 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void iconButton_Click(object sender, RoutedEventArgs e)
        {
            if(ArrowVisibility == Visibility.Visible)
            {
                if(this.isMouseOverButton)
                {
                    this.isMouseOverButton = false;

                    return;
                }

                this.popup.IsOpen = !this.popup.IsOpen;
            }
        }

        #endregion
        #region 팝업 닫는 경우 처리하기 - popup_Closed(sender, e)

        /// <summary>
        /// 팝업 닫는 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void popup_Closed(object sender, EventArgs e)
        {
            Point position = Mouse.GetPosition(this.iconButton);

            if(position.X >= 0 && position.X <= this.iconButton.ActualWidth && position.Y >= 0 && position.Y <= this.iconButton.ActualHeight)
            {
                this.isMouseOverButton = true;
            }
            else
            {
                this.isMouseOverButton = false;
            }
        }

        #endregion
        #region 자식 아이콘 버튼 클릭시 처리하기 - childIconButton_Click(sender, e)

        /// <summary>
        /// 자식 아이콘 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void childIconButton_Click(object sender, RoutedEventArgs e)
        {
            IconButton iconButton = sender as IconButton;

            Value = iconButton.Value;

            this.popup.IsOpen = false;
        }

        #endregion
    }
}