﻿using System;
using System.Collections;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 아이콘 셀렉터 버튼
    /// </summary>
    public partial class IconSelectorButton : UserControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값 변경시 - ValueChanged

        /// <summary>
        /// 값 변경시
        /// </summary>
        public event EventHandler ValueChanged;

        #endregion
        #region 클릭시 - Click

        /// <summary>
        /// 클릭시
        /// </summary>
        public event EventHandler Click;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 항목 소스 의존 속성
        /// </summary>
        public static readonly DependencyProperty ItemsSourceProperty;

        /// <summary>
        /// 항목 패널 의존 속성
        /// </summary>
        public static readonly DependencyProperty ItemsPanelProperty;

        /// <summary>
        /// 아이콘 의존 속성
        /// </summary>
        public static readonly DependencyProperty IconProperty;

        /// <summary>
        /// 화살표 표시 여부 의존 속성
        /// </summary>
        public static readonly DependencyProperty ArrowVisibilityProperty;

        /// <summary>
        /// 그룹명 의존 속성
        /// </summary>
        public static readonly DependencyProperty GroupNameProperty;

        /// <summary>
        /// 값 의존 속성
        /// </summary>
        public static readonly DependencyProperty ValueProperty;

        /// <summary>
        /// 체크 여부 의존 속성
        /// </summary>
        public static readonly DependencyProperty IsCheckedProperty;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 첫번째 로드 여부
        /// </summary>
        private bool firstLoaded = true;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 소스 - ItemsSource

        /// <summary>
        /// 항목 소스
        /// </summary>
        public IEnumerable ItemsSource
        {
            get
            {
                return (IEnumerable)GetValue(ItemsSourceProperty);
            }
            set
            {
                SetValue(ItemsSourceProperty, value);
            }
        }

        #endregion
        #region 항목 패널 - ItemsPanel

        /// <summary>
        /// 항목 패널
        /// </summary>
        public ItemsPanelTemplate ItemsPanel
        {
            get
            {
                return (ItemsPanelTemplate)GetValue(ItemsPanelProperty);
            }
            set
            {
                SetValue(ItemsPanelProperty, value);
            }
        }

        #endregion
        #region 아이콘 - Icon

        /// <summary>
        /// 아이콘
        /// </summary>
        public ImageSource Icon
        {
            get
            {
                return (ImageSource)GetValue(IconProperty);
            }
            set
            {
                SetValue(IconProperty, value);
            }
        }

        #endregion
        #region 화살표 표시 여부 - ArrowVisibility

        /// <summary>
        /// 화살표 표시 여부
        /// </summary>
        public Visibility ArrowVisibility
        {
            get
            {
                return (Visibility)GetValue(ArrowVisibilityProperty);
            }
            set
            {
                SetValue(ArrowVisibilityProperty, value);
            }
        }

        #endregion
        #region 그룹명 - GroupName

        /// <summary>
        /// 그룹명
        /// </summary>
        public string GroupName
        {
            get
            {
                return (string)GetValue(GroupNameProperty);
            }
            set
            {
                SetValue(GroupNameProperty, value);
            }
        }

        #endregion
        #region 값 - Value

        /// <summary>
        /// 값
        /// </summary>
        public string Value
        {
            get
            {
                return (string)GetValue(ValueProperty);
            }
            set
            {
                SetValue(ValueProperty, value);
            }
        }

        #endregion
        #region 체크 여부 - IsChecked

        /// <summary>
        /// 체크 여부
        /// </summary>
        public bool? IsChecked
        {
            get
            {
                return (bool?)GetValue(IsCheckedProperty);
            }
            set
            {
                SetValue(IsCheckedProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - IconSelectorButton()

        /// <summary>
        /// 생성자
        /// </summary>
        static IconSelectorButton()
        {
            ItemsSourceProperty     = UIHelper.RegisterDependencyProperty(null              , false, false, false, true, null                        , "ItemsSource"    , typeof(IEnumerable       ), typeof(IconSelectorButton), null);
            ItemsPanelProperty      = UIHelper.RegisterDependencyProperty(null              , false, false, false, true, null                        , "ItemsPanel"     , typeof(ItemsPanelTemplate), typeof(IconSelectorButton), null);
            IconProperty            = UIHelper.RegisterDependencyProperty(null              , false, false, false, true, null                        , "Icon"           , typeof(ImageSource       ), typeof(IconSelectorButton), null);
            ArrowVisibilityProperty = UIHelper.RegisterDependencyProperty(Visibility.Visible, false, false, false, true, null                        , "ArrowVisibility", typeof(Visibility        ), typeof(IconSelectorButton), null);
            GroupNameProperty       = UIHelper.RegisterDependencyProperty(string.Empty      , false, false, false, true, null                        , "GroupName"      , typeof(string            ), typeof(IconSelectorButton), null);
            ValueProperty           = UIHelper.RegisterDependencyProperty(string.Empty      , false, false, false, true, valuePropertyChangedCallback, "Value"          , typeof(string            ), typeof(IconSelectorButton), null);
            IsCheckedProperty       = UIHelper.RegisterDependencyProperty(false             , false, false, false, true, null                        , "IsChecked"      , typeof(bool?             ), typeof(IconSelectorButton), null);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - IconSelectorButton()

        /// <summary>
        /// 생성자
        /// </summary>
        public IconSelectorButton()
        {
            InitializeComponent();

            UIHelper.SetBinding(this, ItemsSourceProperty, this.itemsControl, ItemsControl.ItemsSourceProperty, BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);
            UIHelper.SetBinding(this, ItemsPanelProperty , this.itemsControl, ItemsControl.ItemsPanelProperty , BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);
            UIHelper.SetBinding(this, IsCheckedProperty  , this.iconButton  , IconButton.IsCheckedProperty    , BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);

            this.popup.IsOpen = true;
            this.popup.IsOpen = false;

            this.Loaded                  += UserControl_Loaded;
            this.iconButton.Click        += iconButton_Click;
            this.iconButton.MouseEnter   += iconButton_MouseEnter;
            this.itemsControl.MouseLeave += itemsControl_MouseLeave;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 값 속성 변경시 콜백 처리하기 - valuePropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 값 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void valuePropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            IconSelectorButton iconSelectorButton = dependencyObject as IconSelectorButton;


            string newValue = e.NewValue == null ? string.Empty : e.NewValue as string;

            IEnumerable enumerable = iconSelectorButton.ItemsSource as IEnumerable;

            if(enumerable != null)
            {
                foreach(object child in enumerable)
                {
                    IconButton childIconButton = child as IconButton;

                    if(childIconButton != null)
                    {
                        try
                        {
                            childIconButton.Click -= iconSelectorButton.childIconButton_Click;

                            if(childIconButton.Value.Equals(newValue))
                            {
                                childIconButton.IsChecked = true;

                                iconSelectorButton.Icon    = childIconButton.Icon;
                                iconSelectorButton.ToolTip = childIconButton.ToolTip;
                            }
                            else
                            {
                                childIconButton.IsChecked = false;
                            }
                        }
                        finally
                        {
                            childIconButton.Click += iconSelectorButton.childIconButton_Click;
                        }
                    }
                }
            }


            iconSelectorButton.FireValueChanged();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 클릭 실행하기 - PerformClick()

        /// <summary>
        /// 클릭 실행하기
        /// </summary>
        public void PerformClick()
        {
            this.iconButton.RaiseEvent(new RoutedEventArgs(ButtonBase.ClickEvent));
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 값 변경시 이벤트 발생시키기 - FireValueChanged()

        /// <summary>
        /// 값 변경시 이벤트 발생시키기
        /// </summary>
        protected void FireValueChanged()
        {
            if(ValueChanged != null)
            {
                ValueChanged(this, EventArgs.Empty);
            }
        }

        #endregion
        #region 클릭시 이벤트 발생시키기 - FireClick()

        /// <summary>
        /// 클릭시 이벤트 발생시키기
        /// </summary>
        protected void FireClick()
        {
            if(Click != null)
            {
                Click(this, EventArgs.Empty);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 사용자 컨트롤 로드시 처리하기 - UserControl_Loaded(object sender, RoutedEventArgs e)

        /// <summary>
        /// 사용자 컨트롤 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            if(this.firstLoaded)
            {
                this.firstLoaded = false;
            }
            else
            {
                return;
            }

            if(ItemsSource != null)
            {
                IEnumerable enumerable = ItemsSource as IEnumerable;

                foreach(object child in enumerable)
                {
                    IconButton childIconButton = child as IconButton;

                    if(childIconButton != null)
                    {
                        childIconButton.Click += childIconButton_Click;
                    }
                }
            }
        }

        #endregion
        #region 아이콘 버튼 마우스 진입시 처리하기 - iconButton_MouseEnter(sender, e)

        /// <summary>
        /// 아이콘 버튼 마우스 진입시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void iconButton_MouseEnter(object sender, MouseEventArgs e)
        {
            if(ArrowVisibility == System.Windows.Visibility.Visible)
            {
                if(this.popup.IsOpen)
                {
                    this.popup.IsOpen = false;
                }

                this.popup.IsOpen = true;
            }
        }

        #endregion
        #region 아이콘 버튼 클릭시 처리하기 - iconButton_Click(sender, e)

        /// <summary>
        /// 아이콘 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void iconButton_Click(object sender, RoutedEventArgs e)
        {
            FireClick();
        }

        #endregion
        #region 자식 아이콘 버튼 클릭시 처리하기 - childIconButton_Click(sender, e)

        /// <summary>
        /// 자식 아이콘 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void childIconButton_Click(object sender, RoutedEventArgs e)
        {
            IconButton iconButton = sender as IconButton;

            if(iconButton.Value == Value)
            {
                this.popup.IsOpen = false;

                if(IsChecked.GetValueOrDefault() == false)
                {
                    FireValueChanged();
                }
            }
            else
            {
                Value = iconButton.Value;

                this.popup.IsOpen = false;
            }
        }

        #endregion
        #region 항목 컨트롤 마우스 이탈시 처리하기 - itemsControl_MouseLeave(sender, e)

        /// <summary>
        /// 항목 컨트롤 마우스 이탈시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void itemsControl_MouseLeave(object sender, MouseEventArgs e)
        {
            this.popup.IsOpen = false;
        }

        #endregion
    }
}