﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace TestProject
{
    /// <summary>
    /// 레이아웃 버튼
    /// </summary>
    public partial class LayoutButton : UserControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 전체 카운트 변경시 - TotalCountChanged

        /// <summary>
        /// 전체 카운트 변경시
        /// </summary>
        public event EventHandler TotalCountChanged;

        #endregion
        #region 카운트 변경시 - CountChanged

        /// <summary>
        /// 카운트 변경시
        /// </summary>
        public event EventHandler CountChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 팝업 너비 의존 속성
        /// </summary>
        public static readonly DependencyProperty PopupWidthProperty;

        /// <summary>
        /// 팝업 높이 의존 속성
        /// </summary>
        public static readonly DependencyProperty PopupHeightProperty;

        /// <summary>
        /// 전체 행 카운트 의존 속성
        /// </summary>
        public static readonly DependencyProperty TotalRowCountProperty;

        /// <summary>
        /// 전체 컬럼 카운트 의존 속성
        /// </summary>
        public static readonly DependencyProperty TotalColumnCountProperty;

        /// <summary>
        /// 카운트 의존 속성
        /// </summary>
        public static readonly DependencyProperty CountProperty;

        /// <summary>
        /// 행 카운트 의존 속성
        /// </summary>
        public static readonly DependencyProperty RowCountProperty;

        /// <summary>
        /// 컬럼 카운트 의존 속성
        /// </summary>
        public static readonly DependencyProperty ColumnCountProperty;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 마우스 버튼 상위 여부
        /// </summary>
        private bool isMouseOverButton = false;

        /// <summary>
        /// 커서
        /// </summary>
        private Cursor cursor;

        /// <summary>
        /// 내부에서 전체 카운트 설정 여부
        /// </summary>
        private bool isInternalTotalCountSet = false;

        #endregion
        
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 팝업 너비 - PopupWidth

        /// <summary>
        /// 팝업 너비
        /// </summary>
        public double PopupWidth
        {
            get
            {
                return (double)GetValue(PopupWidthProperty);
            }
            set
            {
                SetValue(PopupWidthProperty, value);
            }
        }

        #endregion
        #region 팝업 높이 - PopupHeight

        /// <summary>
        /// 팝업 높이
        /// </summary>
        public double PopupHeight
        {
            get
            {
                return (double)GetValue(PopupHeightProperty);
            }
            set
            {
                SetValue(PopupHeightProperty, value);
            }
        }

        #endregion
        #region 전체 행 카운트 - TotalRowCount

        /// <summary>
        /// 전체 행 카운트
        /// </summary>
        public int TotalRowCount
        {
            get
            {
                return (int)GetValue(TotalRowCountProperty);
            }
            set
            {
                SetValue(TotalRowCountProperty, value);
            }
        }

        #endregion
        #region 전체 컬럼 카운트 - TotalColumnCount

        /// <summary>
        /// 전체 컬럼 카운트
        /// </summary>
        public int TotalColumnCount
        {
            get
            {
                return (int)GetValue(TotalColumnCountProperty);
            }
            set
            {
                SetValue(TotalColumnCountProperty, value);
            }
        }

        #endregion
        #region 카운트 - Count

        /// <summary>
        /// 카운트
        /// </summary>
        public string Count
        {
            get
            {
                return (string)GetValue(CountProperty);
            }
            set
            {
                SetValue(CountProperty, value);
            }
        }

        #endregion
        #region 행 카운트 - RowCount

        /// <summary>
        /// 행 카운트
        /// </summary>
        public int RowCount
        {
            get
            {
                return (int)GetValue(RowCountProperty);
            }
            set
            {
                SetValue(RowCountProperty, value);
            }
        }

        #endregion
        #region 컬럼 카운트 - ColumnCount

        /// <summary>
        /// 컬럼 카운트
        /// </summary>
        public int ColumnCount
        {
            get
            {
                return (int)GetValue(ColumnCountProperty);
            }
            set
            {
                SetValue(ColumnCountProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - LayoutButton()

        /// <summary>
        /// 생성자
        /// </summary>
        static LayoutButton()
        {
            PopupWidthProperty       = UIHelper.RegisterDependencyProperty(double.NaN  , false, false, false, true, null                                   , "PopupWidth"      , typeof(double), typeof(LayoutButton), null);
            PopupHeightProperty      = UIHelper.RegisterDependencyProperty(double.NaN  , false, false, false, true, null                                   , "PopupHeight"     , typeof(double), typeof(LayoutButton), null);
            TotalRowCountProperty    = UIHelper.RegisterDependencyProperty(10          , false, false, false, true, totalRowCountPropertyChangedCallback   , "TotalRowCount"   , typeof(int)   , typeof(LayoutButton), null);
            TotalColumnCountProperty = UIHelper.RegisterDependencyProperty(10          , false, false, false, true, totalColumnCountPropertyChangedCallback, "TotalColumnCount", typeof(int)   , typeof(LayoutButton), null);
            CountProperty            = UIHelper.RegisterDependencyProperty(string.Empty, false, false, false, true, countPropertyChangedCallback           , "Count"           , typeof(string), typeof(LayoutButton), null);
            RowCountProperty         = UIHelper.RegisterDependencyProperty(0           , false, false, false, true, null                                   , "RowCount"        , typeof(int)   , typeof(LayoutButton), null);
            ColumnCountProperty      = UIHelper.RegisterDependencyProperty(0           , false, false, false, true, null                                   , "ColumnCount"     , typeof(int)   , typeof(LayoutButton), null);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - LayoutButton()

        /// <summary>
        /// 생성자
        /// </summary>
        public LayoutButton()
        {
            InitializeComponent();

            this.popup.IsOpen = true;
            this.popup.IsOpen = false;

            this.button.Click   += button_Click;
            this.popup.Closed   += popup_Closed;
            this.grid.MouseMove += grid_MouseMove;

            this.grid.AddHandler(Rectangle.MouseDownEvent, new MouseButtonEventHandler(rectangle_MouseDown));

            this.thumb.DragStarted   += thumb_DragStarted;
            this.thumb.DragDelta     += thumb_DragDelta;
            this.thumb.DragCompleted += thumb_DragCompleted;

            SetGrid(10, 10);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 전체 행 카운트 속성 변경시 콜백 처리하기 - totalRowCountPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 전체 행 카운트 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void totalRowCountPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            LayoutButton layoutButton = dependencyObject as LayoutButton;

            
            int newValue = (int)e.NewValue;

            if(!layoutButton.isInternalTotalCountSet)
            {
                layoutButton.SetGrid(newValue, layoutButton.TotalColumnCount);
            }

            layoutButton.isInternalTotalCountSet = false;


            layoutButton.FireTotalCountChanged();
        }

        #endregion
        #region 전체 컬럼 카운트 속성 변경시 콜백 처리하기 - totalColumnCountPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 전체 컬럼 카운트 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void totalColumnCountPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            LayoutButton layoutButton = dependencyObject as LayoutButton;

            
            int newValue = (int)e.NewValue;

            if(!layoutButton.isInternalTotalCountSet)
            {
                layoutButton.SetGrid(layoutButton.TotalRowCount, newValue);
            }

            layoutButton.isInternalTotalCountSet = false;


            layoutButton.FireTotalCountChanged();
        }

        #endregion
        #region 카운트 속성 변경시 콜백 처리하기 - countPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 카운트 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void countPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            LayoutButton layoutButton = dependencyObject as LayoutButton;

            
            layoutButton.FireCountChanged();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 전체 카운트 변경시 이벤트 발생시키기 - FireTotalCountChanged()

        /// <summary>
        /// 전체 카운트 변경시 이벤트 발생시키기
        /// </summary>
        protected void FireTotalCountChanged()
        {
            if(TotalCountChanged != null)
            {
                TotalCountChanged(this, EventArgs.Empty);
            }
        }

        #endregion
        #region 카운트 변경시 이벤트 발생시키기 - FireCountChanged()

        /// <summary>
        /// 카운트 변경시 이벤트 발생시키기
        /// </summary>
        protected void FireCountChanged()
        {
            if(CountChanged != null)
            {
                CountChanged(this, EventArgs.Empty);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private
        ////////////////////////////////////////////////////////////////////// Event

        #region 버튼 클릭시 처리하기 - button_Click(sender, e)

        /// <summary>
        /// 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void button_Click(object sender, RoutedEventArgs e)
        {
            if(this.isMouseOverButton)
            {
                this.isMouseOverButton = false;

                return;
            }

            this.popup.IsOpen = !this.popup.IsOpen;
        }

        #endregion
        #region 팝업 닫는 경우 처리하기 - popup_Closed(sender, e)

        /// <summary>
        /// 팝업 닫는 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void popup_Closed(object sender, EventArgs e)
        {
            Point position = Mouse.GetPosition(this.button);

            if(position.X >= 0 && position.X <= this.button.ActualWidth && position.Y >= 0 && position.Y <= this.button.ActualHeight)
            {
                this.isMouseOverButton = true;
            }
            else
            {
                this.isMouseOverButton = false;
            }
        }

        #endregion
        #region 그리드 마우스 이동시 처리하기 - grid_MouseMove(sender, e)

        /// <summary>
        /// 그리드 마우스 이동시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void grid_MouseMove(object sender, MouseEventArgs e)
        {
            HitTestResult result = VisualTreeHelper.HitTest(this.grid, e.GetPosition(this.grid));

            Rectangle currentRectangle = result.VisualHit as Rectangle;

            if(currentRectangle == null)
            {
                this.textBlock.Text = string.Empty;
            }
            else
            {
                int currentRowIndex    = Grid.GetRow(currentRectangle);
                int currentColumnIndex = Grid.GetColumn(currentRectangle);

                this.textBlock.Text = string.Format("{0}×{1}", currentRowIndex + 1, currentColumnIndex + 1);

                for(int i = 0; i < this.grid.Children.Count; i++)
                {
                    Rectangle rectangle = this.grid.Children[i] as Rectangle;

                    int rowIndex    = Grid.GetRow(rectangle);
                    int columnIndex = Grid.GetColumn(rectangle);

                    if(rowIndex <= currentRowIndex && columnIndex <= currentColumnIndex)
                    {
                        rectangle.Stroke = Brushes.Gold;
                    }
                    else
                    {
                        rectangle.Stroke = Brushes.Black;
                    }
                }
            }
        }

        #endregion
        #region 사각형 마우스 DOWN 처리하기 - rectangle_MouseDown(sender, e)

        /// <summary>
        /// 사각형 마우스 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void rectangle_MouseDown(object sender, MouseButtonEventArgs e)
        {
            HitTestResult result = VisualTreeHelper.HitTest(this.grid, e.GetPosition(this.grid));

            Rectangle currentRectangle = result.VisualHit as Rectangle;

            if(currentRectangle != null)
            {
                int currentRowIndex    = Grid.GetRow(currentRectangle);
                int currentColumnIndex = Grid.GetColumn(currentRectangle);

                RowCount    = currentRowIndex + 1;
                ColumnCount = currentColumnIndex + 1;
                Count       = string.Format("{0}×{1}", RowCount, ColumnCount);

                this.textBlock.Text = string.Empty;

                for(int i = 0; i < this.grid.Children.Count; i++)
                {
                    Rectangle rectangle = this.grid.Children[i] as Rectangle;

                    rectangle.Stroke = Brushes.Black;
                }

                this.grid.RowDefinitions.Clear();
                this.grid.ColumnDefinitions.Clear();
                this.grid.Children.Clear();

                this.grid.Width  = double.NaN;
                this.grid.Height = double.NaN;

                SetGrid(TotalRowCount, TotalColumnCount);

                PopupWidth  = double.NaN;
                PopupHeight = double.NaN;

                this.popup.IsOpen = false;
            }
        }

        #endregion
        #region Thumb 드래그 시작시 처리하기 - thumb_DragStarted(sender, e)

        /// <summary>
        /// Thumb 드래그 시작시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void thumb_DragStarted(object sender, DragStartedEventArgs e)
        {
            this.cursor = Cursor;

            Cursor = Cursors.SizeNWSE;


            this.grid.Width  = this.grid.ActualWidth;
            this.grid.Height = this.grid.ActualHeight;


            this.grid.RowDefinitions.Clear();
            this.grid.ColumnDefinitions.Clear();
            this.grid.Children.Clear();
        }

        #endregion
        #region Thumb 드래그 변경시 처리하기 - thumb_DragDelta(sender, e)

        /// <summary>
        /// Thumb 드래그 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void thumb_DragDelta(object sender, DragDeltaEventArgs e)
        {
            double adjustX = PopupWidth  + e.HorizontalChange;
            double adjustY = PopupHeight + e.VerticalChange;

            adjustX = adjustX > this.popup.MinWidth  ? adjustX : this.popup.MinWidth;
            adjustY = adjustY > this.popup.MinHeight ? adjustY : this.popup.MinHeight;

            PopupWidth  = adjustX;
            PopupHeight = adjustY;
        }

        #endregion
        #region Thumb 드래그 완료시 처리하기 - thumb_DragCompleted(sender, e)

        /// <summary>
        /// Thumb 드래그 완료시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void thumb_DragCompleted(object sender, DragCompletedEventArgs e)
        {
            Cursor = this.cursor;


            int rowCount    = Convert.ToInt32(Math.Ceiling(PopupHeight / 22d));
            int columnCount = Convert.ToInt32(Math.Ceiling(PopupWidth  / 22d));


            this.isInternalTotalCountSet = true;

            TotalRowCount = rowCount;

            this.isInternalTotalCountSet = true;

            TotalColumnCount = columnCount;


            this.grid.Width  = double.NaN;
            this.grid.Height = double.NaN;


            SetGrid(rowCount, columnCount);


            PopupWidth  = double.NaN;
            PopupHeight = double.NaN;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////// Function

        #region 그리드 설정하기 - SetGrid(rowCount, columnCount)

        /// <summary>
        /// 그리드 설정하기
        /// </summary>
        /// <param name="rowCount">행 카운트</param>
        /// <param name="columnCount">컬럼 카운트</param>
        private void SetGrid(int rowCount, int columnCount)
        {
            for(int i = 0; i < rowCount; i++)
            {
                RowDefinition rowDefinition = new RowDefinition();

                rowDefinition.Height = new GridLength(1d, GridUnitType.Star);

                this.grid.RowDefinitions.Add(rowDefinition);
            }

            for(int i = 0; i < columnCount; i++)
            {
                ColumnDefinition columnDefinition = new ColumnDefinition();

                columnDefinition.Width = new GridLength(1d, GridUnitType.Star);

                this.grid.ColumnDefinitions.Add(columnDefinition);
            }

            for(int y = 0; y < rowCount; y++)
            {
                for(int x = 0; x < columnCount; x++)
                {
                    Rectangle rectangle = new Rectangle();

                    rectangle.HorizontalAlignment = HorizontalAlignment.Center;
                    rectangle.VerticalAlignment   = VerticalAlignment.Center;
                    rectangle.Margin              = new Thickness(1);
                    rectangle.Width               = 20d;
                    rectangle.Height              = 20d;
                    rectangle.Stroke              = Brushes.Black;
                    rectangle.StrokeThickness     = 1d;
                    rectangle.Fill                = Brushes.White;

                    this.grid.Children.Add(rectangle);

                    Grid.SetRow   (rectangle, y);
                    Grid.SetColumn(rectangle, x);
                }
            }
        }

        #endregion
    }
}