﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Input;

namespace TestProject
{
    /// <summary>
    /// 멀티 콤보 박스
    /// </summary>
    public partial class MultiComboBox : UserControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 항목들 소스 변경시 - ItemsSourceChanged

        /// <summary>
        /// 항목들 소스 변경시
        /// </summary>
        public event EventHandler ItemsSourceChanged;

        #endregion
        #region 값들 변경시 - ValuesChanged

        /// <summary>
        /// 값들 변경시
        /// </summary>
        public event EventHandler ValuesChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 팝업 너비 의존 속성
        /// </summary>
        public static readonly DependencyProperty PopupWidthProperty;

        /// <summary>
        /// 팝업 높이 의존 속성
        /// </summary>
        public static readonly DependencyProperty PopupHeightProperty;

        /// <summary>
        /// 항목들 소스 의존 속성
        /// </summary>
        public static readonly DependencyProperty ItemsSourceProperty;

        /// <summary>
        /// 값들 의존 속성
        /// </summary>
        public static readonly DependencyProperty ValuesProperty;

        /// <summary>
        /// 텍스트들 의존 속성
        /// </summary>
        public static readonly DependencyProperty TextsProperty;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 마우스 버튼 상위 여부
        /// </summary>
        private bool isMouseOverButton = false;

        /// <summary>
        /// 커서
        /// </summary>
        private Cursor cursor;

        /// <summary>
        /// 팝업 첫번째 오픈 여부
        /// </summary>
        private bool isPopupFirstOpen = true;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 버튼 - Button

        /// <summary>
        /// 버튼
        /// </summary>
        public Button Button
        {
            get
            {
                return this.button;
            }
        }

        #endregion
        #region 멀티 리스트 박스 - MultiListBox

        /// <summary>
        /// 멀티 리스트 박스
        /// </summary>
        public MultiListBox MultiListBox
        {
            get
            {
                return this.multiListBox;
            }
        }

        #endregion

        #region 팝업 너비 - PopupWidth

        /// <summary>
        /// 팝업 너비
        /// </summary>
        public double PopupWidth
        {
            get
            {
                return (double)GetValue(PopupWidthProperty);
            }
            set
            {
                SetValue(PopupWidthProperty, value);
            }
        }

        #endregion
        #region 팝업 높이 - PopupHeight

        /// <summary>
        /// 팝업 높이
        /// </summary>
        public double PopupHeight
        {
            get
            {
                return (double)GetValue(PopupHeightProperty);
            }
            set
            {
                SetValue(PopupHeightProperty, value);
            }
        }

        #endregion
        #region 항목들 소스 - ItemsSource

        /// <summary>
        /// 항목들 소스
        /// </summary>
        public IEnumerable ItemsSource
        {
            get
            {
                return (IEnumerable)GetValue(ItemsSourceProperty);
            }
            set
            {
                SetValue(ItemsSourceProperty, value);
            }
        }

        #endregion
        #region 값들 - Values

        /// <summary>
        /// 값들
        /// </summary>
        public string Values
        {
            get
            {
                return (string)GetValue(ValuesProperty);
            }
            set
            {
                SetValue(ValuesProperty, value);
            }
        }

        #endregion
        #region 텍스트들 - Texts

        /// <summary>
        /// 텍스트들
        /// </summary>
        public string Texts
        {
            get
            {
                return (string)GetValue(TextsProperty);
            }
            set
            {
                SetValue(TextsProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - MultiComboBox()

        /// <summary>
        /// 생성자
        /// </summary>
        static MultiComboBox()
        {
            PopupWidthProperty  = UIHelper.RegisterDependencyProperty(150d        , false, false, false, true, null                              , "PopupWidth" , typeof(double)     , typeof(MultiComboBox), null);
            PopupHeightProperty = UIHelper.RegisterDependencyProperty(100d        , false, false, false, true, null                              , "PopupHeight", typeof(double)     , typeof(MultiComboBox), null);
            ItemsSourceProperty = UIHelper.RegisterDependencyProperty(null        , false, false, false, true, itemsSourcePropertyChangedCallback, "ItemsSource", typeof(IEnumerable), typeof(MultiComboBox), null);
            ValuesProperty      = UIHelper.RegisterDependencyProperty(string.Empty, false, false, false, true, valuesPropertyChangedCallback     , "Values"     , typeof(string)     , typeof(MultiComboBox), null);
            TextsProperty       = UIHelper.RegisterDependencyProperty(string.Empty, false, false, false, true, null                              , "Texts"      , typeof(string)     , typeof(MultiComboBox), null);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MultiComboBox()

        /// <summary>
        /// 생성자
        /// </summary>
        public MultiComboBox()
        {
            InitializeComponent();

            UIHelper.SetBinding(this       , ItemsSourceProperty, this.multiListBox, MultiListBox.ItemsSourceProperty, BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);
            UIHelper.SetBinding(this       , ValuesProperty     , this.multiListBox, MultiListBox.ValuesProperty     , BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);
            UIHelper.SetBinding(this       , TextsProperty      , this.multiListBox, MultiListBox.TextsProperty      , BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);
            UIHelper.SetBinding(this.button, ContentProperty    , this.multiListBox, MultiListBox.TextsProperty      , BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);

            this.popup.IsOpen = true;
            this.popup.IsOpen = false;

            this.button.Click               += button_Click;
            this.popup.Opened               += popup_Opened;
            this.popup.Closed               += popup_Closed;
            this.showAllCheckBox.Checked    += showAllCheckBox_Checked;
            this.showAllCheckBox.Unchecked  += showAllCheckBox_Checked;
            this.multiListBox.ValuesChanged += multiListBox_ValuesChanged;
            this.topButton.Click            += topButton_Click;
            this.bottomButton.Click         += bottomButton_Click;
            this.topMostButton.Click        += topMostButton_Click;
            this.thumb.DragStarted          += thumb_DragStarted;
            this.thumb.DragDelta            += thumb_DragDelta;
            this.thumb.DragCompleted        += thumb_DragCompleted;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 항목들 소스 속성 변경시 콜백 처리하기 - itemsSourcePropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 항목들 소스 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void itemsSourcePropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            MultiComboBox multiComboBox = dependencyObject as MultiComboBox;


            multiComboBox.isPopupFirstOpen = true;


            multiComboBox.Values = string.Empty;


            multiComboBox.FireItemsSourceChanged();
        }

        #endregion
        #region 값들 속성 변경시 콜백 처리하기 - valuesPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 값들 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void valuesPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            MultiComboBox multiComboBox = dependencyObject as MultiComboBox;


            multiComboBox.FireValuesChanged();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 모든 항목 체크하기 - CheckAll()

        /// <summary>
        /// 모든 항목 체크하기
        /// </summary>
        public void CheckAll()
        {
            this.multiListBox.CheckAll();
        }

        #endregion
        #region 모든 항목 체크 취소하기 - UncheckAll()

        /// <summary>
        /// 모든 항목 체크 취소하기
        /// </summary>
        public void UncheckAll()
        {
            this.multiListBox.UncheckAll();
        }

        #endregion
        #region 값 항목 리스트 구하기 - GetValueItemList()

        /// <summary>
        /// 값 항목 리스트 구하기
        /// </summary>
        /// <returns>값 항목 리스트</returns>
        public List<ListItem> GetValueItemList()
        {
            return this.multiListBox.GetValueItemList();
        }

        #endregion
        #region 체크 항목 가장 위쪽으로 이동시키기 - MoveTopMost()

        /// <summary>
        /// 체크 항목 가장 위쪽으로 이동시키기
        /// </summary>
        public void MoveTopMost()
        {
            this.multiListBox.MoveTopMost();
        }

        #endregion
        #region 선택 항목 위쪽으로 이동시키기 - MoveTop()

        /// <summary>
        /// 선택 항목 위쪽으로 이동시키기
        /// </summary>
        public void MoveTop()
        {
            this.multiListBox.MoveTop();
        }

        #endregion
        #region 선택 항목 아래쪽으로 이동시키기 - MoveBottom()

        /// <summary>
        /// 선택 항목 아래쪽으로 이동시키기
        /// </summary>
        public void MoveBottom()
        {
            this.multiListBox.MoveBottom();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 항목들 소스 변경시 이벤트 발생시키기 - FireItemsSourceChanged()

        /// <summary>
        /// 항목들 소스 변경시 이벤트 발생시키기
        /// </summary>
        protected void FireItemsSourceChanged()
        {
            if(ItemsSourceChanged != null)
            {
                ItemsSourceChanged(this, EventArgs.Empty);
            }
        }

        #endregion
        #region 값들 변경시 이벤트 발생시키기 - FireValuesChanged()

        /// <summary>
        /// 값들 변경시 이벤트 발생시키기
        /// </summary>
        protected void FireValuesChanged()
        {
            if(ValuesChanged != null)
            {
                ValuesChanged(this, EventArgs.Empty);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 버튼 클릭시 처리하기 - button_Click(sender, e)

        /// <summary>
        /// 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void button_Click(object sender, RoutedEventArgs e)
        {
            if(this.isMouseOverButton)
            {
                this.isMouseOverButton = false;

                return;
            }

            this.popup.IsOpen = !this.popup.IsOpen;
        }

        #endregion
        #region 팝업 여는 경우 처리하기 - popup_Opened(sender, e)

        /// <summary>
        /// 팝업 여는 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void popup_Opened(object sender, EventArgs e)
        {
            ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

            if(collection == null || collection.Count == 0)
            {
                PopupHeight = 150d;

                return;
            }


            if(this.isPopupFirstOpen)
            {
                ListBoxItem listBoxItem = this.multiListBox.ItemContainerGenerator.ContainerFromIndex(0) as ListBoxItem;

                if(listBoxItem != null)
                {
                    double height = listBoxItem.ActualHeight;

                    if(!height.Equals(double.NaN))
                    {
                        PopupHeight = Math.Min(collection.Count * height, 20 * height) + 82;
                    }
                }
                else
                {
                    PopupHeight = Math.Min(collection.Count * 18, 20 * 18) + 82;
                }

                this.isPopupFirstOpen = false;
            }


            List<ListItem> list = this.multiListBox.GetValueItemList();

            if(list == null || list.Count == 0)
            {
                return;
            }

            this.multiListBox.ScrollIntoView(list[0]);
        }

        #endregion
        #region 팝업 닫는 경우 처리하기 - popup_Closed(sender, e)

        /// <summary>
        /// 팝업 닫는 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void popup_Closed(object sender, EventArgs e)
        {
            Point position = Mouse.GetPosition(this.button);

            if(position.X >= 0 && position.X <= this.button.ActualWidth && position.Y >= 0 && position.Y <= this.button.ActualHeight)
            {
                this.isMouseOverButton = true;
            }
            else
            {
                this.isMouseOverButton = false;
            }
        }

        #endregion
        #region Show All 체크 박스 체크시 처리하기 - showAllCheckBox_Checked(sender, e)

        /// <summary>
        /// Show All 체크 박스 체크시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void showAllCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            if(this.showAllCheckBox.IsChecked.GetValueOrDefault())
            {
                this.multiListBox.CheckAll();
            }
            else
            {
                this.multiListBox.UncheckAll();
            }
        }

        #endregion
        #region 멀티 리스트 박스 값 목록 변경시 처리하기 - multiListBox_ValuesChanged(sender, e)

        /// <summary>
        /// 멀티 리스트 박스 값 목록 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void multiListBox_ValuesChanged(object sender, EventArgs e)
        {
            SetShowAllCheckBoxValue();
        }

        #endregion
        #region 선택 항목 위쪽으로 이동 버튼 클릭시 처리하기 - topButton_Click(sender, e)

        /// <summary>
        /// 선택 항목 위쪽으로 이동 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void topButton_Click(object sender, RoutedEventArgs e)
        {
            this.multiListBox.MoveTop();
        }

        #endregion
        #region 선택 항목 아래쪽으로 이동 버튼 클릭시 처리하기 - bottomButton_Click(sender, e)

        /// <summary>
        /// 선택 항목 아래쪽으로 이동 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생</param>
        /// <param name="e">이벤트 인자</param>
        private void bottomButton_Click(object sender, RoutedEventArgs e)
        {
            this.multiListBox.MoveBottom();
        }

        #endregion
        #region 체크 항목 가장 위쪽으로 이동 버튼 클릭시 처리하기 - topMostButton_Click(sender, e)

        /// <summary>
        /// 체크 항목 가장 위쪽으로 이동 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void topMostButton_Click(object sender, RoutedEventArgs e)
        {
            this.multiListBox.MoveTopMost();
        }

        #endregion
        #region Thumb 드래그 시작시 처리하기 - thumb_DragStarted(sender, e)

        /// <summary>
        /// Thumb 드래그 시작시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void thumb_DragStarted(object sender, DragStartedEventArgs e)
        {
            this.cursor = Cursor;

            Cursor = Cursors.SizeNWSE;
        }

        #endregion
        #region Thumb 드래그 변경시 처리하기 - thumb_DragDelta(sender, e)

        /// <summary>
        /// Thumb 드래그 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void thumb_DragDelta(object sender, DragDeltaEventArgs e)
        {
            double adjustX = PopupWidth  + e.HorizontalChange;
            double adjustY = PopupHeight + e.VerticalChange;

            adjustX = adjustX > this.popup.MinWidth  ? adjustX : this.popup.MinWidth;
            adjustY = adjustY > this.popup.MinHeight ? adjustY : this.popup.MinHeight;

            PopupWidth  = adjustX;
            PopupHeight = adjustY;
        }

        #endregion
        #region Thumb 드래그 완료시 처리하기 - thumb_DragCompleted(sender, e)

        /// <summary>
        /// Thumb 드래그 완료시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void thumb_DragCompleted(object sender, DragCompletedEventArgs e)
        {
            Cursor = this.cursor;
        }

        #endregion
        #region Show All 체크박스 값 설정하기 - SetShowAllCheckBoxValue()

        /// <summary>
        /// Show All 체크박스 값 설정하기
        /// </summary>
        private void SetShowAllCheckBoxValue()
        {
            if(this.showAllCheckBox != null)
            {
                try
                {
                    this.showAllCheckBox.Checked   -= showAllCheckBox_Checked;
                    this.showAllCheckBox.Unchecked -= showAllCheckBox_Checked;

                    ObservableCollection<ListItem> collection = this.multiListBox.ItemsSource as ObservableCollection<ListItem>;

                    if(collection == null || collection.Count == 0)
                    {
                        this.showAllCheckBox.IsChecked = false;

                        return;
                    }

                    this.showAllCheckBox.IsChecked = true;

                    foreach(ListItem item in collection)
                    {
                        if(!item.IsChecked)
                        {
                            this.showAllCheckBox.IsChecked = false;

                            break;
                        }
                    }
                }
                finally
                {
                    this.showAllCheckBox.Checked   += showAllCheckBox_Checked;
                    this.showAllCheckBox.Unchecked += showAllCheckBox_Checked;
                }
            }
        }

        #endregion
    }
}