﻿using System.ComponentModel;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 리스트 항목
    /// </summary>
    public class ListItem : INotifyPropertyChanged
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 - PropertyChanged

        /// <summary>
        /// 속성 변경시
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 체크 여부
        /// </summary>
        private bool isChecked;

        /// <summary>
        /// 아이콘
        /// </summary>
        private ImageSource icon;

        /// <summary>
        /// 값
        /// </summary>
        private string value;

        /// <summary>
        /// 텍스트
        /// </summary>
        private string text;

        /// <summary>
        /// 태그
        /// </summary>
        private object tag;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 체크 여부 - IsChecked

        /// <summary>
        /// 체크 여부
        /// </summary>
        public bool IsChecked
        {
            get
            {
                return this.isChecked;
            }
            set
            {
                if(this.isChecked == value)
                {
                    return;
                }

                this.isChecked = value;

                OnPropertyChanged("IsChecked");
            }
        }

        #endregion
        #region 아이콘 - Icon

        /// <summary>
        /// 아이콘
        /// </summary>
        public ImageSource Icon
        {
            get
            {
                return this.icon;
            }
            set
            {
                if(this.icon == value)
                {
                    return;
                }

                this.icon = value;

                OnPropertyChanged("Icon");
            }
        }

        #endregion
        #region 값 - Value

        /// <summary>
        /// 값
        /// </summary>
        public string Value
        {
            get
            {
                return this.value;
            }
            set
            {
                if(this.value == value)
                {
                    return;
                }

                this.value = value;

                OnPropertyChanged("Value");
            }
        }

        #endregion
        #region 텍스트 - Text

        /// <summary>
        /// 텍스트
        /// </summary>
        public string Text
        {
            get
            {
                return this.text;
            }
            set
            {
                if(this.text == value)
                {
                    return;
                }

                this.text = value;

                OnPropertyChanged("Text");
            }
        }

        #endregion
        #region 태그 - Tag

        /// <summary>
        /// 태그
        /// </summary>
        public object Tag
        {
            get
            {
                return this.tag;
            }
            set
            {
                if(this.tag == value)
                {
                    return;
                }

                this.tag = value;

                OnPropertyChanged("Tag");
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MultiItem()

        /// <summary>
        /// 생성자
        /// </summary>
        public ListItem()
        {
            this.isChecked = false;
            this.icon      = null;
            this.value     = null;
            this.text      = null;
            this.tag       = null;
        }

        #endregion
        #region 생성자 - MultiItem(value)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="value">값</param>
        public ListItem(string value)
        {
            this.isChecked = false;
            this.icon      = null;
            this.value     = value;
            this.text      = value;
            this.tag       = null;
        }

        #endregion
        #region 생성자 - MultiItem(value, text)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="value">값</param>
        /// <param name="text">텍스트</param>
        public ListItem(string value, string text)
        {
            this.isChecked = false;
            this.icon      = null;
            this.value     = value;
            this.text      = text;
            this.tag       = null;
        }

        #endregion
        #region 생성자 - MultiItem(isChecked, value, text)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="isChecked">체크 여부</param>
        /// <param name="value">값</param>
        /// <param name="text">텍스트</param>
        public ListItem(bool isChecked, string value, string text)
        {
            this.isChecked = isChecked;
            this.icon      = null;
            this.value     = value;
            this.text      = text;
            this.tag       = null;
        }

        #endregion
        #region 생성자 - MultiItem(isChecked, icon, value, text)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="isChecked">체크 여부</param>
        /// <param name="icon">아이콘</param>
        /// <param name="value">값</param>
        /// <param name="text">텍스트</param>
        public ListItem(bool isChecked, ImageSource icon, string value, string text)
        {
            this.isChecked = isChecked;
            this.icon      = icon;
            this.value     = value;
            this.text      = text;
            this.tag       = null;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 속성 변경시 처리하기 - OnPropertyChanged(proertyName)

        /// <summary>
        /// 속성 변경시 처리하기
        /// </summary>
        /// <param name="proertyName">속성명</param>
        protected void OnPropertyChanged(string proertyName)
        {
            if(PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(proertyName));
            }
        }

        #endregion
    }
}