﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Input;

namespace TestProject
{
    /// <summary>
    /// 우선 순위 콤보 박스
    /// </summary>
    public partial class PriorityComboBox : UserControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값들 변경시 - ValuesChanged

        /// <summary>
        /// 값들 목록 변경시
        /// </summary>
        public event EventHandler ValuesChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region Field

        /// <summary>
        /// 항목 소스 의존 속성
        /// </summary>
        public static readonly DependencyProperty ItemsSourceProperty;

        /// <summary>
        /// 팝업 너비 의존 속성
        /// </summary>
        public static readonly DependencyProperty PopupWidthProperty;

        /// <summary>
        /// 팝업 높이 의존 속성
        /// </summary>
        public static readonly DependencyProperty PopupHeightProperty;

        /// <summary>
        /// 값들 의존 속성
        /// </summary>
        public static readonly DependencyProperty ValuesProperty;

        /// <summary>
        /// 텍스트들 의존 속성
        /// </summary>
        public static readonly DependencyProperty TextsProperty;

        /// <summary>
        /// 우선 순위 표시 여부 의존 속성
        /// </summary>
        public static readonly DependencyProperty ShowPriorityProperty;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 값들 내부 설정 여부
        /// </summary>
        private bool isValuesInternalSetting = false;

        /// <summary>
        /// 마우스 버튼 상위 여부
        /// </summary>
        private bool isMouseOverButton = false;

        /// <summary>
        /// 커서
        /// </summary>
        private Cursor cursor;

        /// <summary>
        /// 팝업 첫번째 오픈 여부
        /// </summary>
        private bool isPopupFirstOpen = true;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 버튼 - Button

        /// <summary>
        /// 버튼
        /// </summary>
        public Button Button
        {
            get
            {
                return this.button;
            }
        }

        #endregion
        #region 우선 순위 리스트 박스 - PriorityListBox

        /// <summary>
        /// 우선 순위 리스트 박스
        /// </summary>
        public PriorityListBox PriorityListBox
        {
            get
            {
                return this.priorityListBox;
            }
        }

        #endregion

        #region 항목 소스 - ItemsSource

        /// <summary>
        /// 항목 소스
        /// </summary>
        public IEnumerable ItemsSource
        {
            get
            {
                return (IEnumerable)GetValue(ItemsSourceProperty);
            }
            set
            {
                SetValue(ItemsSourceProperty, value);
            }
        }

        #endregion
        #region 팝업 너비 - PopupWidth

        /// <summary>
        /// 팝업 너비
        /// </summary>
        public double PopupWidth
        {
            get
            {
                return (double)GetValue(PopupWidthProperty);
            }
            set
            {
                SetValue(PopupWidthProperty, value);
            }
        }

        #endregion
        #region 팝업 높이 - PopupHeight

        /// <summary>
        /// 팝업 높이
        /// </summary>
        public double PopupHeight
        {
            get
            {
                return (double)GetValue(PopupHeightProperty);
            }
            set
            {
                SetValue(PopupHeightProperty, value);
            }
        }

        #endregion
        #region 값들 - Values

        /// <summary>
        /// 값들
        /// </summary>
        public string Values
        {
            get
            {
                return (string)GetValue(ValuesProperty);
            }
            set
            {
                SetValue(ValuesProperty, value);
            }
        }

        #endregion
        #region 텍스트들 - Texts

        /// <summary>
        /// 텍스트들
        /// </summary>
        public string Texts
        {
            get
            {
                return (string)GetValue(TextsProperty);
            }
            set
            {
                SetValue(TextsProperty, value);
            }
        }

        #endregion
        #region 우선 순위 표시 여부 - ShowPriority

        /// <summary>
        /// 우선 순위 표시 여부
        /// </summary>
        public bool ShowPriority
        {
            get
            {
                return (bool)GetValue(ShowPriorityProperty);
            }
            set
            {
                SetValue(ShowPriorityProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - PriorityComboBox()

        /// <summary>
        /// 생성자
        /// </summary>
        static PriorityComboBox()
        {
            ItemsSourceProperty  = UIHelper.RegisterDependencyProperty(null        , false, false, false, true, itemsSourcePropertyChangedCallback , "ItemsSource" , typeof(IEnumerable), typeof(PriorityComboBox), null);
            PopupWidthProperty   = UIHelper.RegisterDependencyProperty(150d        , false, false, false, true, null                               , "PopupWidth"  , typeof(double)     , typeof(PriorityComboBox), null);
            PopupHeightProperty  = UIHelper.RegisterDependencyProperty(100d        , false, false, false, true, null                               , "PopupHeight" , typeof(double)     , typeof(PriorityComboBox), null);
            ValuesProperty       = UIHelper.RegisterDependencyProperty(string.Empty, false, false, false, true, valuesPropertyChangedCallback      , "Values"      , typeof(string)     , typeof(PriorityComboBox), null);
            TextsProperty        = UIHelper.RegisterDependencyProperty(string.Empty, false, false, false, true, null                               , "Texts"       , typeof(string)     , typeof(PriorityComboBox), null);
            ShowPriorityProperty = UIHelper.RegisterDependencyProperty(true        , false, false, false, true, showPriorityPropertyChangedCallback, "ShowPriority", typeof(bool)       , typeof(PriorityComboBox), null);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - PriorityComboBox()

        /// <summary>
        /// 생성자
        /// </summary>
        public PriorityComboBox()
        {
            InitializeComponent();

            UIHelper.SetBinding(this, ShowPriorityProperty, this.priorityListBox, PriorityListBox.ShowPriorityProperty, BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);

            this.popup.IsOpen = true;
            this.popup.IsOpen = false;

            this.button.Click                  += button_Click;
            this.popup.Opened                  += popup_Opened;
            this.popup.Closed                  += popup_Closed;
            this.showAllCheckBox.Checked       += showAllCheckBox_Checked;
            this.showAllCheckBox.Unchecked     += showAllCheckBox_Checked;
            this.priorityListBox.ValuesChanged += priorityListBox_ValuesChanged;
            this.topButton.Click               += topButton_Click;
            this.bottomButton.Click            += bottomButton_Click;
            this.topMostButton.Click           += topMostButton_Click;
            this.okButton.Click                += okButton_Click;
            this.cancelButton.Click            += cancelButton_Click;
            this.thumb.DragStarted             += thumb_DragStarted;
            this.thumb.DragDelta               += thumb_DragDelta;
            this.thumb.DragCompleted           += thumb_DragCompleted;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 항목들 소스 속성 변경시 콜백 처리하기 - itemsSourcePropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 항목들 소스 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void itemsSourcePropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            PriorityComboBox priorityComboBox = dependencyObject as PriorityComboBox;


            priorityComboBox.isPopupFirstOpen = true;


            priorityComboBox.PriorityListBox.ItemsSource = priorityComboBox.ItemsSource;


            if(priorityComboBox.Values != string.Empty)
            {
                priorityComboBox.isValuesInternalSetting = true;
    
                priorityComboBox.Values = string.Empty;

                priorityComboBox.Texts = string.Empty;
            }
        }

        #endregion
        #region 값들 속성 변경시 콜백 처리하기 - valuesPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 값들 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void valuesPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            PriorityComboBox priorityComboBox = dependencyObject as PriorityComboBox;


            string newValue = e.NewValue == null ? string.Empty : e.NewValue as string;

            if(!priorityComboBox.isValuesInternalSetting)
            {
                if(priorityComboBox.ItemsSource != priorityComboBox.PriorityListBox.ItemsSource)
                {
                    priorityComboBox.PriorityListBox.ItemsSource = priorityComboBox.ItemsSource;
                }

                priorityComboBox.PriorityListBox.SetChecked(newValue);

                priorityComboBox.PriorityListBox.Values = priorityComboBox.PriorityListBox.GetValues();
            }

            priorityComboBox.isValuesInternalSetting = false;


            priorityComboBox.Texts = priorityComboBox.PriorityListBox.Texts;


            priorityComboBox.FireValuesChanged();
        }

        #endregion
        #region 우선 순위 표시 여부 속성 변경시 콜백 처리하기 - showPriorityPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 우선 순위 표시 여부 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void showPriorityPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            PriorityComboBox priorityComboBox = dependencyObject as PriorityComboBox;


            priorityComboBox.Values = priorityComboBox.PriorityListBox.Values;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 모든 항목 체크하기 - CheckAll()

        /// <summary>
        /// 모든 항목 체크하기
        /// </summary>
        public void CheckAll()
        {
            if(ItemsSource != this.priorityListBox.ItemsSource)
            {
                this.priorityListBox.ItemsSource = ItemsSource;
            }


            this.priorityListBox.CheckAll();


            Values = priorityListBox.Values;
            Texts  = priorityListBox.Texts;
        }

        #endregion
        #region 모든 항목 체크 취소하기 - UncheckAll()

        /// <summary>
        /// 모든 항목 체크 취소하기
        /// </summary>
        public void UncheckAll()
        {
            if(ItemsSource != this.priorityListBox.ItemsSource)
            {
                this.priorityListBox.ItemsSource = ItemsSource;
            }


            this.priorityListBox.UncheckAll();


            Values = priorityListBox.Values;
            Texts  = priorityListBox.Texts;
        }

        #endregion
        #region 값 항목 리스트 구하기 - GetValueItemList()

        /// <summary>
        /// 값 항목 리스트 구하기
        /// </summary>
        /// <returns>값 항목 리스트</returns>
        public List<PriorityItem> GetValueItemList()
        {
            if(ItemsSource != this.priorityListBox.ItemsSource)
            {
                this.priorityListBox.ItemsSource = ItemsSource;
            }


            return this.priorityListBox.GetValueItemList();
        }

        #endregion
        #region 체크 항목 가장 위쪽으로 이동시키기 - MoveTopMost()

        /// <summary>
        /// 체크 항목 가장 위쪽으로 이동시키기
        /// </summary>
        public void MoveTopMost()
        {
            if(ItemsSource != this.priorityListBox.ItemsSource)
            {
                this.priorityListBox.ItemsSource = ItemsSource;
            }


            this.priorityListBox.MoveTopMost();


            Values = priorityListBox.Values;
            Texts  = priorityListBox.Texts;
        }

        #endregion
        #region 선택 항목 위쪽으로 이동시키기 - MoveTop()

        /// <summary>
        /// 선택 항목 위쪽으로 이동시키기
        /// </summary>
        public void MoveTop()
        {
            if(ItemsSource != this.priorityListBox.ItemsSource)
            {
                this.priorityListBox.ItemsSource = ItemsSource;
            }


            this.priorityListBox.MoveTop();


            Values = priorityListBox.Values;
            Texts  = priorityListBox.Texts;
        }

        #endregion
        #region 선택 항목 아래쪽으로 이동시키기 - MoveBottom()

        /// <summary>
        /// 선택 항목 아래쪽으로 이동시키기
        /// </summary>
        public void MoveBottom()
        {
            if(ItemsSource != this.priorityListBox.ItemsSource)
            {
                this.priorityListBox.ItemsSource = ItemsSource;
            }


            this.priorityListBox.MoveBottom();


            Values = priorityListBox.Values;
            Texts  = priorityListBox.Texts;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 값들 변경시 이벤트 발생시키기 - FireValuesChanged()

        /// <summary>
        /// 값들 변경시 이벤트 발생시키기
        /// </summary>
        protected void FireValuesChanged()
        {
            if(ValuesChanged != null)
            {
                ValuesChanged(this, EventArgs.Empty);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private
        ////////////////////////////////////////////////////////////////////// Event

        #region 버튼 클릭시 처리하기 - button_Click(sender, e)

        /// <summary>
        /// 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void button_Click(object sender, RoutedEventArgs e)
        {
            if(this.isMouseOverButton)
            {
                this.isMouseOverButton = false;

                return;
            }


            this.popup.IsOpen = !this.popup.IsOpen;
        }

        #endregion
        #region 팝업 오픈시 처리하기 - popup_Opened(sender, e)

        /// <summary>
        /// 팝업 오픈시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void popup_Opened(object sender, EventArgs e)
        {
            ObservableCollection<PriorityItem> collection = ItemsSource as ObservableCollection<PriorityItem>;

            if(collection == null || collection.Count == 0)
            {
                PopupHeight = 150d;

                return;
            }


            ObservableCollection<PriorityItem> newCollection = GetNewCollection(collection);

            this.priorityListBox.ItemsSource = newCollection;


            SetShowAllCheckBoxValue();


            if(this.isPopupFirstOpen)
            {
                ListBoxItem listBoxItem = this.priorityListBox.ItemContainerGenerator.ContainerFromIndex(0) as ListBoxItem;

                if(listBoxItem != null)
                {
                    double height = listBoxItem.ActualHeight;

                    if(!height.Equals(double.NaN))
                    {
                        PopupHeight = Math.Min(newCollection.Count * height, 20 * height) + 82;
                    }
                }
                else
                {
                    PopupHeight = Math.Min(collection.Count * 16, 20 * 16) + 82;
                }

                this.isPopupFirstOpen = false;
            }


            List<PriorityItem> list = this.priorityListBox.GetValueItemList();

            if(list == null || list.Count == 0)
            {
                return;
            }

            this.priorityListBox.ScrollIntoView(list[0]);
        }

        #endregion
        #region 팝업 닫는 경우 처리하기 - popup_Closed(sender, e)

        /// <summary>
        /// 팝업 닫는 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void popup_Closed(object sender, EventArgs e)
        {
            Point position = Mouse.GetPosition(this.button);

            if(position.X >= 0 && position.X <= this.button.ActualWidth && position.Y >= 0 && position.Y <= this.button.ActualHeight)
            {
                this.isMouseOverButton = true;
            }
            else
            {
                this.isMouseOverButton = false;
            }

            ClearShowAllCheckBoxValue();
        }

        #endregion
        #region Show All 체크 박스 체크시 처리하기 - showAllCheckBox_Checked(sender, e)

        /// <summary>
        /// Show All 체크 박스 체크시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void showAllCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            if(this.showAllCheckBox.IsChecked.GetValueOrDefault())
            {
                this.priorityListBox.CheckAll();
            }
            else
            {
                this.priorityListBox.UncheckAll();
            }
        }

        #endregion
        #region 컬럼 리스트 박스 값 목록 변경시 처리하기 - columnListBox_ValuesChanged(sender, e)

        /// <summary>
        /// 컬럼 리스트 박스 값 목록 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void priorityListBox_ValuesChanged(object sender, EventArgs e)
        {
            SetShowAllCheckBoxValue();
        }

        #endregion
        #region 선택 항목 위쪽으로 이동 버튼 클릭시 처리하기 - topButton_Click(sender, e)

        /// <summary>
        /// 선택 항목 위쪽으로 이동 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void topButton_Click(object sender, RoutedEventArgs e)
        {
            this.priorityListBox.MoveTop();
        }

        #endregion
        #region 선택 항목 아래쪽으로 이동 버튼 클릭시 처리하기 - bottomButton_Click(sender, e)

        /// <summary>
        /// 선택 항목 아래쪽으로 이동 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void bottomButton_Click(object sender, RoutedEventArgs e)
        {
            this.priorityListBox.MoveBottom();
        }

        #endregion
        #region 체크 항목 가장 위쪽으로 이동 버튼 클릭시 처리하기 - topMostButton_Click(sender, e)

        /// <summary>
        /// 체크 항목 가장 위쪽으로 이동 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void topMostButton_Click(object sender, RoutedEventArgs e)
        {
            this.priorityListBox.MoveTopMost();
        }

        #endregion
        #region Ok 버튼 클릭시 처리하기 - okButton_Click(sender, e)

        /// <summary>
        /// Ok 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void okButton_Click(object sender, RoutedEventArgs e)
        {
            ItemsSource = this.priorityListBox.ItemsSource;


            if(Values != this.priorityListBox.Values)
            {
                this.isValuesInternalSetting = true;

                Values = this.priorityListBox.Values;
            }


            this.popup.IsOpen = false;
        }

        #endregion
        #region Cancel 버튼 클릭시 처리하기 - cancelButton_Click(sender, e)

        /// <summary>
        /// Cancel 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void cancelButton_Click(object sender, RoutedEventArgs e)
        {
            this.popup.IsOpen = false;
        }

        #endregion
        #region Thumb 드래그 시작시 처리하기 - thumb_DragStarted(sender, e)

        /// <summary>
        /// Thumb 드래그 시작시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void thumb_DragStarted(object sender, DragStartedEventArgs e)
        {
            this.cursor = Cursor;

            Cursor = Cursors.SizeNWSE;
        }

        #endregion
        #region Thumb 드래그 변경시 처리하기 - thumb_DragDelta(sender, e)

        /// <summary>
        /// Thumb 드래그 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void thumb_DragDelta(object sender, DragDeltaEventArgs e)
        {
            double adjustX = PopupWidth  + e.HorizontalChange;
            double adjustY = PopupHeight + e.VerticalChange;

            adjustX = adjustX > this.popup.MinWidth  ? adjustX : this.popup.MinWidth;
            adjustY = adjustY > this.popup.MinHeight ? adjustY : this.popup.MinHeight;

            PopupWidth  = adjustX;
            PopupHeight = adjustY;
        }

        #endregion
        #region Thumb 드래그 완료시 처리하기 - thumb_DragCompleted(sender, e)

        /// <summary>
        /// Thumb 드래그 완료시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void thumb_DragCompleted(object sender, DragCompletedEventArgs e)
        {
            Cursor = this.cursor;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////// Function

        #region 새 컬렉션 구하기 - GetNewCollection(sourceCollection)

        /// <summary>
        /// 새 컬렉션 구하기
        /// </summary>
        /// <param name="sourceCollection">소스 컬렉션</param>
        /// <returns>새 컬렉션</returns>
        private ObservableCollection<PriorityItem> GetNewCollection(ObservableCollection<PriorityItem> sourceCollection)
        {
            ObservableCollection<PriorityItem> targetCollection = new ObservableCollection<PriorityItem>();

            if(sourceCollection == null || sourceCollection.Count == 0)
            {
                return targetCollection;
            }


            foreach(PriorityItem item in sourceCollection)
            {
                targetCollection.Add(item.GetNewItem());
            }


            return targetCollection;
        }

        #endregion
        #region Show All 체크 박스 값 설정하기 - SetShowAllCheckBoxValue()

        /// <summary>
        /// Show All 체크 박스 값 설정하기
        /// </summary>
        private void SetShowAllCheckBoxValue()
        {
            if(this.showAllCheckBox != null)
            {
                try
                {
                    this.showAllCheckBox.Checked   -= showAllCheckBox_Checked;
                    this.showAllCheckBox.Unchecked -= showAllCheckBox_Checked;


                    ObservableCollection<PriorityItem> collection = this.priorityListBox.ItemsSource as ObservableCollection<PriorityItem>;

                    if(collection == null || collection.Count == 0)
                    {
                        this.showAllCheckBox.IsChecked = false;

                        return;
                    }


                    foreach(PriorityItem item in collection)
                    {
                        if(!item.IsChecked)
                        {
                            this.showAllCheckBox.IsChecked = false;

                            return;
                        }
                    }

                    this.showAllCheckBox.IsChecked = true;
                }
                finally
                {
                    this.showAllCheckBox.Checked   += showAllCheckBox_Checked;
                    this.showAllCheckBox.Unchecked += showAllCheckBox_Checked;
                }
            }
        }

        #endregion
        #region Show All 체크 박스 값 지우기 - ClearShowAllCheckBoxValue()

        /// <summary>
        /// Show All 체크 박스 값 지우기
        /// </summary>
        private void ClearShowAllCheckBoxValue()
        {
            if(this.showAllCheckBox != null)
            {
                try
                {
                    this.showAllCheckBox.Checked   -= showAllCheckBox_Checked;
                    this.showAllCheckBox.Unchecked -= showAllCheckBox_Checked;

                    this.showAllCheckBox.IsChecked = false;
                }
                finally
                {
                    this.showAllCheckBox.Checked   += showAllCheckBox_Checked;
                    this.showAllCheckBox.Unchecked += showAllCheckBox_Checked;
                }
            }
        }

        #endregion
    }
}