﻿using System.ComponentModel;

namespace TestProject
{
    /// <summary>
    /// 우선 순위 항목
    /// </summary>
    public class PriorityItem : INotifyPropertyChanged
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 - PropertyChanged

        /// <summary>
        /// 속성 변경시
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 체크 여부
        /// </summary>
        private bool isChecked;

        /// <summary>
        /// 값
        /// </summary>
        private string value;

        /// <summary>
        /// 텍스트
        /// </summary>
        private string text;

        /// <summary>
        /// 내림차순 여부
        /// </summary>
        private bool isDescending;

        /// <summary>
        /// 태그
        /// </summary>
        private object tag;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 체크 여부 - IsChecked

        /// <summary>
        /// 체크 여부
        /// </summary>
        public bool IsChecked
        {
            get
            {
                return this.isChecked;
            }
            set
            {
                if(this.isChecked == value)
                {
                    return;
                }

                this.isChecked = value;

                FirePropertyChanged("IsChecked");
            }
        }

        #endregion
        #region 값 - Value

        /// <summary>
        /// 값
        /// </summary>
        public string Value
        {
            get
            {
                return this.value;
            }
            set
            {
                if(this.value == value)
                {
                    return;
                }

                this.value = value;

                FirePropertyChanged("Value");
            }
        }

        #endregion
        #region 텍스트 - Text

        /// <summary>
        /// 텍스트
        /// </summary>
        public string Text
        {
            get
            {
                return this.text;
            }
            set
            {
                if(this.text == value)
                {
                    return;
                }

                this.text = value;

                FirePropertyChanged("Text");
            }
        }

        #endregion
        #region 내림차순 여부 - IsDescending

        /// <summary>
        /// 내림차순 여부
        /// </summary>
        public bool IsDescending
        {
            get
            {
                return this.isDescending;
            }
            set
            {
                if(this.isDescending == value)
                {
                    return;
                }

                this.isDescending = value;

                FirePropertyChanged("IsDescending");
            }
        }

        #endregion
        #region 태그 - Tag

        /// <summary>
        /// 태그
        /// </summary>
        public object Tag
        {
            get
            {
                return this.tag;
            }
            set
            {
                if(this.tag == value)
                {
                    return;
                }

                this.tag = value;

                FirePropertyChanged("Tag");
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - PriorityItem()

        /// <summary>
        /// 생성자
        /// </summary>
        public PriorityItem()
        {
            this.isChecked    = false;
            this.value        = null;
            this.text         = null;
            this.isDescending = false;
            this.tag          = null;
        }

        #endregion
        #region 생성자 - PriorityItem(value)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="value">값</param>
        public PriorityItem(string value)
        {
            this.isChecked    = false;
            this.value        = value;
            this.text         = value;
            this.isDescending = false;
            this.tag          = null;
        }

        #endregion
        #region 생성자 - PriorityItem(value, isDescending)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="value">값</param>
        /// <param name="isDescending">내림차순 여부</param>
        public PriorityItem(string value, bool isDescending)
        {
            this.isChecked    = false;
            this.value        = value;
            this.text         = value;
            this.isDescending = isDescending;
            this.tag          = null;
        }

        #endregion
        #region 생성자 - PriorityItem(value, text)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="value">값</param>
        /// <param name="text">텍스트</param>
        public PriorityItem(string value, string text)
        {
            this.isChecked    = false;
            this.value        = value;
            this.text         = text;
            this.isDescending = false;
            this.tag          = null;
        }

        #endregion
        #region 생성자 - PriorityItem(isChecked, value)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="isChecked">체크 여부</param>
        /// <param name="value">값</param>
        public PriorityItem(bool isChecked, string value)
        {
            this.isChecked    = isChecked;
            this.value        = value;
            this.text         = value;
            this.isDescending = false;
            this.tag          = null;
        }

        #endregion
        #region 생성자 - PriorityItem(isChecked, value, isDescending)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="isChecked">체크 여부</param>
        /// <param name="value">값</param>
        /// <param name="isDescending">내림차순 여부</param>
        public PriorityItem(bool isChecked, string value, bool isDescending)
        {
            this.isChecked    = isChecked;
            this.value        = value;
            this.text         = value;
            this.isDescending = isDescending;
            this.tag          = null;
        }

        #endregion
        #region 생성자 - PriorityItem(isChecked, value, text)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="isChecked">체크 여부</param>
        /// <param name="value">값</param>
        /// <param name="text">텍스트</param>
        public PriorityItem(bool isChecked, string value, string text)
        {
            this.isChecked    = isChecked;
            this.value        = value;
            this.text         = text;
            this.isDescending = false;
            this.tag          = null;
        }

        #endregion
        #region 생성자 - PriorityItem(isChecked, value, text, isDescending)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="isChecked">체크 여부</param>
        /// <param name="value">값</param>
        /// <param name="text">텍스트</param>
        /// <param name="isDescending">내림차순 여부</param>
        public PriorityItem(bool isChecked, string value, string text, bool isDescending)
        {
            this.isChecked    = isChecked;
            this.value        = value;
            this.text         = text;
            this.isDescending = isDescending;
            this.tag          = null;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 문자열 구하기 - ToString()

        /// <summary>
        /// 문자열 구하기
        /// </summary>
        /// <returns>문자열</returns>
        public override string ToString()
        {
            return this.text;
        }

        #endregion
        #region 새 항목 구하기 - GetNewItem()

        /// <summary>
        /// 새 항목 구하기
        /// </summary>
        /// <returns>새 항목</returns>
        public PriorityItem GetNewItem()
        {
            PriorityItem item = new PriorityItem();

            item.isChecked    = this.isChecked;
            item.value        = this.value;
            item.text         = this.text;
            item.isDescending = this.isDescending;
            item.tag          = this.tag;

            return item;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 속성 변경시 이벤트 발생시키기 - FirePropertyChanged(proertyName)

        /// <summary>
        /// 속성 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="proertyName">속성명</param>
        protected void FirePropertyChanged(string proertyName)
        {
            if(PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(proertyName));
            }
        }

        #endregion
    }
}