﻿using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Input;

namespace TestProject
{
    /// <summary>
    /// 싱글 콤보 박스
    /// </summary>
    public partial class SingleComboBox : UserControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 소스 변경시 - ItemsSourceChanged

        /// <summary>
        /// 항목 소스 변경시
        /// </summary>
        public event EventHandler ItemsSourceChanged;

        #endregion
        #region 값 변경시 - ValueChanged

        /// <summary>
        /// 값 변경시
        /// </summary>
        public event EventHandler ValueChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 항목 소스 의존 속성
        /// </summary>
        public static readonly DependencyProperty ItemsSourceProperty;

        /// <summary>
        /// 팝업 너비 의존 속성
        /// </summary>
        public static readonly DependencyProperty PopupWidthProperty;

        /// <summary>
        /// 팝업 높이 의존 속성
        /// </summary>
        public static readonly DependencyProperty PopupHeightProperty;

        /// <summary>
        /// 값 의존 속성
        /// </summary>
        public static readonly DependencyProperty ValueProperty;

        /// <summary>
        /// 텍스트 의존 속성
        /// </summary>
        public static readonly DependencyProperty TextProperty;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 마우스 버튼 상위 여부
        /// </summary>
        private bool isMouseOverButton = false;

        /// <summary>
        /// 커서
        /// </summary>
        private Cursor cursor;

        /// <summary>
        /// 팝업 첫번째 오픈 여부
        /// </summary>
        private bool isPopupFirstOpen = true;

        #endregion
        
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 소스 - ItemsSource

        /// <summary>
        /// 항목 소스
        /// </summary>
        public IEnumerable ItemsSource
        {
            get
            {
                return (IEnumerable)GetValue(ItemsSourceProperty);
            }
            set
            {
                SetValue(ItemsSourceProperty, value);
            }
        }

        #endregion
        #region 팝업 너비 - PopupWidth

        /// <summary>
        /// 팝업 너비
        /// </summary>
        public double PopupWidth
        {
            get
            {
                return (double)GetValue(PopupWidthProperty);
            }
            set
            {
                SetValue(PopupWidthProperty, value);
            }
        }

        #endregion
        #region 팝업 높이 - PopupHeight

        /// <summary>
        /// 팝업 높이
        /// </summary>
        public double PopupHeight
        {
            get
            {
                return (double)GetValue(PopupHeightProperty);
            }
            set
            {
                SetValue(PopupHeightProperty, value);
            }
        }

        #endregion
        #region 값 - Value

        /// <summary>
        /// 값
        /// </summary>
        public string Value
        {
            get
            {
                return (string)GetValue(ValueProperty);
            }
            set
            {
                SetValue(ValueProperty, value);
            }
        }

        #endregion
        #region 텍스트 - Text

        /// <summary>
        /// 텍스트
        /// </summary>
        public string Text
        {
            get
            {
                return (string)GetValue(TextProperty);
            }
            set
            {
                SetValue(TextProperty, value);
            }
        }

        #endregion

        #region 버튼 - Button

        /// <summary>
        /// 버튼
        /// </summary>
        public Button Button
        {
            get
            {
                return this.button;
            }
        }

        #endregion
        #region 싱글 리스트 박스 - SingleListBox

        /// <summary>
        /// 싱글 리스트 박스
        /// </summary>
        public SingleListBox SingleListBox
        {
            get
            {
                return this.singleListBox;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - SingleComboBox()

        /// <summary>
        /// 생성자
        /// </summary>
        static SingleComboBox()
        {
            ItemsSourceProperty = UIHelper.RegisterDependencyProperty(null        , false, false, false, true, itemsSourcePropertyChangedCallback, "ItemsSource", typeof(IEnumerable), typeof(SingleComboBox), null);
            PopupWidthProperty  = UIHelper.RegisterDependencyProperty(150d        , false, false, false, true, null                              , "PopupWidth" , typeof(double)     , typeof(SingleComboBox), null);
            PopupHeightProperty = UIHelper.RegisterDependencyProperty(100d        , false, false, false, true, null                              , "PopupHeight", typeof(double)     , typeof(SingleComboBox), null);
            ValueProperty       = UIHelper.RegisterDependencyProperty(string.Empty, false, false, false, true, valuePropertyChangedCallback      , "Value"      , typeof(string)     , typeof(SingleComboBox), null);
            TextProperty        = UIHelper.RegisterDependencyProperty(string.Empty, false, false, false, true, null                              , "Text"       , typeof(string)     , typeof(SingleComboBox), null);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SingleComboBox()

        /// <summary>
        /// 생성자
        /// </summary>
        public SingleComboBox()
        {
            InitializeComponent();

            UIHelper.SetBinding(this, ItemsSourceProperty, this.singleListBox, SingleListBox.ItemsSourceProperty, BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);
            UIHelper.SetBinding(this, ValueProperty      , this.singleListBox, SingleListBox.ValueProperty      , BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);
            UIHelper.SetBinding(this, TextProperty       , this.singleListBox, SingleListBox.TextProperty       , BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);

            this.popup.IsOpen = true;
            this.popup.IsOpen = false;

            this.button.Click              += button_Click;
            this.popup.Opened              += popup_Opened;
            this.popup.Closed              += popup_Closed;
            this.singleListBox.ItemMouseUp += singleListBox_ItemMouseUp;
            this.thumb.DragStarted         += thumb_DragStarted;
            this.thumb.DragDelta           += thumb_DragDelta;
            this.thumb.DragCompleted       += thumb_DragCompleted;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 항목 소스 속성 변경시 콜백 처리하기 - itemsSourcePropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 항목 소스 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void itemsSourcePropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            SingleComboBox singleComboBox = dependencyObject as SingleComboBox;


            singleComboBox.isPopupFirstOpen = true;


            singleComboBox.FireItemsSourceChanged();
        }

        #endregion
        #region 값 속성 변경시 콜백 처리하기 - valuePropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 값 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void valuePropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            SingleComboBox singleComboBox = dependencyObject as SingleComboBox;


            singleComboBox.FireValueChanged();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 항목 소스 변경시 이벤트 발생시키기 - FireItemsSourceChanged()

        /// <summary>
        /// 항목 소스 변경시 이벤트 발생시키기
        /// </summary>
        protected void FireItemsSourceChanged()
        {
            if(ItemsSourceChanged != null)
            {
                ItemsSourceChanged(this, EventArgs.Empty);
            }
        }

        #endregion
        #region 값 변경시 이벤트 발생시키기 - FireValueChanged()

        /// <summary>
        /// 값 변경시 이벤트 발생시키기
        /// </summary>
        protected void FireValueChanged()
        {
            if(ValueChanged != null)
            {
                ValueChanged(this, EventArgs.Empty);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 버튼 클릭시 처리하기 - button_Click(sender, e)

        /// <summary>
        /// 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void button_Click(object sender, RoutedEventArgs e)
        {
            if(this.isMouseOverButton)
            {
                this.isMouseOverButton = false;

                return;
            }

            this.popup.IsOpen = !this.popup.IsOpen;
        }

        #endregion
        #region 팝업 여는 경우 처리하기 - popup_Opened(sender, e)

        /// <summary>
        /// 팝업 여는 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void popup_Opened(object sender, EventArgs e)
        {
            ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

            if(collection == null || collection.Count == 0)
            {
                PopupHeight = 100;

                return;
            }


            if(this.isPopupFirstOpen)
            {
                ListBoxItem listBoxItem = this.singleListBox.ItemContainerGenerator.ContainerFromIndex(0) as ListBoxItem;

                if(listBoxItem != null)
                {
                    double height = listBoxItem.ActualHeight;

                    if(!height.Equals(double.NaN))
                    {
                        PopupHeight = Math.Min(collection.Count * height, 20 * height) + 30;
                    }
                }
                else
                {
                    PopupHeight = Math.Min(collection.Count * 18, 20 * 18) + 30;
                }

                this.isPopupFirstOpen = false;
            }


            ListItem item = this.singleListBox.SelectedItem as ListItem;

            if(item == null)
            {
                return;
            }

            this.singleListBox.ScrollIntoView(item);
        }

        #endregion
        #region 팝업 닫는 경우 처리하기 - popup_Closed(sender, e)

        /// <summary>
        /// 팝업 닫는 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void popup_Closed(object sender, EventArgs e)
        {
            Point position = Mouse.GetPosition(this.button);

            if(position.X >= 0 && position.X <= this.button.ActualWidth && position.Y >= 0 && position.Y <= this.button.ActualHeight)
            {
                this.isMouseOverButton = true;
            }
            else
            {
                this.isMouseOverButton = false;
            }
        }

        #endregion
        #region 싱글 리스트 박스 항목 마우스 UP 처리하기 - singleListBox_ItemMouseUp(sender, e)

        /// <summary>
        /// 싱글 리스트 박스 항목 마우스 UP 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void singleListBox_ItemMouseUp(object sender, EventArgs e)
        {
            this.popup.IsOpen = false;
        }

        #endregion
        #region Thumb 드래그 시작시 처리하기 - thumb_DragStarted(sender, e)

        /// <summary>
        /// Thumb 드래그 시작시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void thumb_DragStarted(object sender, DragStartedEventArgs e)
        {
            this.cursor = Cursor;

            Cursor = Cursors.SizeNWSE;
        }

        #endregion
        #region Thumb 드래그 변경시 처리하기 - thumb_DragDelta(sender, e)

        /// <summary>
        /// Thumb 드래그 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void thumb_DragDelta(object sender, DragDeltaEventArgs e)
        {
            double adjustX = PopupWidth  + e.HorizontalChange;
            double adjustY = PopupHeight + e.VerticalChange;

            adjustX = adjustX > this.popup.MinWidth  ? adjustX : this.popup.MinWidth;
            adjustY = adjustY > this.popup.MinHeight ? adjustY : this.popup.MinHeight;

            PopupWidth  = adjustX;
            PopupHeight = adjustY;
        }

        #endregion
        #region Thumb 드래그 완료시 처리하기 - thumb_DragCompleted(sender, e)

        /// <summary>
        /// Thumb 드래그 완료시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void thumb_DragCompleted(object sender, DragCompletedEventArgs e)
        {
            Cursor = this.cursor;
        }

        #endregion
    }
}