﻿using System;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace TestProject
{
    /// <summary>
    /// 싱글 리스트 박스
    /// </summary>
    public partial class SingleListBox : ListBox
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값 변경시 - ValueChanged

        /// <summary>
        /// 값 변경시
        /// </summary>
        public event EventHandler ValueChanged;

        #endregion
        #region 항목 마우스 UP시 - ItemMouseUp

        /// <summary>
        /// 항목 마우스 UP시
        /// </summary>
        public event EventHandler ItemMouseUp;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 값 의존 속성
        /// </summary>
        public static readonly DependencyProperty ValueProperty;

        /// <summary>
        /// 텍스트 의존 속성
        /// </summary>
        public static readonly DependencyProperty TextProperty;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 값 변경시 이벤트 발생 가능 여부
        /// </summary>
        private bool canFireValueChanged = true;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값 - Value

        /// <summary>
        /// 값
        /// </summary>
        public string Value
        {
            get
            {
                return (string)GetValue(ValueProperty);
            }
            set
            {
                SetValue(ValueProperty, value);
            }
        }

        #endregion
        #region 텍스트 - Text

        /// <summary>
        /// 텍스트
        /// </summary>
        public string Text
        {
            get
            {
                return (string)GetValue(TextProperty);
            }
            set
            {
                SetValue(TextProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - SingleListBox()

        /// <summary>
        /// 생성자
        /// </summary>
        static SingleListBox()
        {
            ValueProperty = UIHelper.RegisterDependencyProperty(string.Empty, false, false, false, true, valuePropertyChangedCallback, "Value", typeof(string), typeof(SingleListBox), null);
            TextProperty  = UIHelper.RegisterDependencyProperty(string.Empty, false, false, false, true, null                        , "Text" , typeof(string), typeof(SingleListBox), null);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SingleListBox()

        /// <summary>
        /// 생성자
        /// </summary>
        public SingleListBox()
        {
            InitializeComponent();

            SelectionChanged += listBox_SelectionChanged;

            AddHandler(TextBlock.MouseDownEvent, new MouseButtonEventHandler(itemTextBlock_MouseDown));
            AddHandler(TextBlock.MouseUpEvent  , new MouseButtonEventHandler(itemTextBlock_MouseUp  ));
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 값 속성 변경시 콜백 처리하기 - valuePropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 값 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void valuePropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            SingleListBox singleListBox = dependencyObject as SingleListBox;


            string newValue = e.NewValue == null ? string.Empty : e.NewValue as string;

            singleListBox.SetValue(newValue);


            string text = singleListBox.GetText();

            singleListBox.Text = text;


            singleListBox.FireValueChanged();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 구하기 - GetItem()

        /// <summary>
        /// 항목 구하기
        /// </summary>
        /// <returns>항목</returns>
        public ListItem GetItem()
        {
            ListItem item = SelectedItem as ListItem;

            return item;
        }

        #endregion
        #region 항목 구하기 - GetItem(index)

        /// <summary>
        /// 항목 구하기
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <returns>항목</returns>
        public ListItem GetItem(int index)
        {
            ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

            if(collection == null || collection.Count == 0)
            {
                return null;
            }

            if(index < 0 || index > collection.Count - 1)
            {
                return null;
            }

            ListItem item = collection[index];

            return item;
        }

        #endregion
        #region 값 포함 여부 구하기 - Contains(value)

        /// <summary>
        /// 값 포함 여부 구하기
        /// </summary>
        /// <param name="value">값</param>
        /// <returns>값 포함 여부</returns>
        public bool Contains(string value)
        {
            ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

            if(collection == null || collection.Count == 0)
            {
                return false;
            }

            foreach(ListItem item in collection)
            {
                if(item.Value.Equals(value))
                {
                    return true;
                }
            }

            return false;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 값 변경시 이벤트 발생시키기 - FireValueChanged()

        /// <summary>
        /// 값 변경시 이벤트 발생시키기
        /// </summary>
        protected void FireValueChanged()
        {
            if(ValueChanged != null)
            {
                ValueChanged(this, EventArgs.Empty);
            }
        }

        #endregion
        #region 항목 마우스 UP시 이벤트 발생시키기 - FireItemMouseUp()

        /// <summary>
        /// 항목 마우스 UP시 이벤트 발생시키기
        /// </summary>
        protected void FireItemMouseUp()
        {
            if(ItemMouseUp != null)
            {
                ItemMouseUp(this, EventArgs.Empty);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private
        ////////////////////////////////////////////////////////////////////// Event

        #region 리스트 박스 선택 변경시 처리하기 - listBox_SelectionChanged(sender, e)

        /// <summary>
        /// 리스트 박스 선택 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void listBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if(this.canFireValueChanged)
            {
                ListItem item = SelectedItem as ListItem;

                if(item == null)
                {
                    Value = string.Empty;

                    return;
                }

                Value = item.Value;
            }
        }

        #endregion
        #region 항목 텍스트 블럭 마우스 DOWN 처리하기 - itemTextBlock_MouseDown(sender, e)

        /// <summary>
        /// 항목 텍스트 블럭 마우스 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void itemTextBlock_MouseDown(object sender, MouseButtonEventArgs e)
        {
            this.canFireValueChanged = false;
        }

        #endregion
        #region 항목 텍스트 블럭 마우스 UP 처리하기 - itemTextBlock_MouseUp(sender, e)

        /// <summary>
        /// 항목 텍스트 블럭 마우스 UP 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void itemTextBlock_MouseUp(object sender, MouseButtonEventArgs e)
        {
            this.canFireValueChanged = true;

            Value = GetValue();

            FireItemMouseUp();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////// Function

        #region 값 설정하기 - SetValue(value)

        /// <summary>
        /// 값 설정하기
        /// </summary>
        /// <param name="value">값</param>
        private void SetValue(string value)
        {
            try
            {
                SelectionChanged -= listBox_SelectionChanged;

                ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

                if(collection == null || collection.Count == 0)
                {
                    return;
                }

                if(string.IsNullOrEmpty(value))
                {
                    SelectedItem = null;

                    return;
                }

                foreach(ListItem item in collection)
                {
                    if(value.Equals(item.Value))
                    {
                        SelectedItem = item;

                        return;
                    }
                }

                SelectedItem = null;
            }
            finally
            {
                SelectionChanged += listBox_SelectionChanged;
            }
        }

        #endregion
        #region 값 구하기 - GetValue()

        /// <summary>
        /// 값 구하기
        /// </summary>
        /// <returns>값</returns>
        private string GetValue()
        {
            if(SelectedItem == null)
            {
                return string.Empty;
            }

            ListItem item = SelectedItem as ListItem;

            if(item == null)
            {
                return string.Empty;
            }
            else
            {
                return item.Value;
            }
        }

        #endregion
        #region 텍스트 구하기 - GetText()

        /// <summary>
        /// 텍스트 구하기
        /// </summary>
        /// <returns>텍스트</returns>
        private string GetText()
        {
            if(SelectedItem == null)
            {
                return string.Empty;
            }

            ListItem item = SelectedItem as ListItem;

            if(item == null)
            {
                return string.Empty;
            }
            else
            {
                return item.Text;
            }
        }

        #endregion
    }
}